import { NgModule, LOCALE_ID, ModuleWithProviders } from '@angular/core';
import { CommonModule, registerLocaleData } from '@angular/common';
import localePt from '@angular/common/locales/pt';
import localePtExtra from '@angular/common/locales/extra/pt';
import { RouterModule } from '@angular/router';
import { FlexLayoutModule } from '@angular/flex-layout';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MatExpansionModule } from '@angular/material/expansion';
import { MatIconModule } from '@angular/material/icon';
import { MatMenuModule } from '@angular/material/menu';
import { MatListModule } from '@angular/material/list';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatTableModule } from '@angular/material/table';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MAT_DATE_LOCALE, MAT_DATE_FORMATS, DateAdapter } from '@angular/material/core';
import { MatDatepickerIntl } from '@angular/material/datepicker';
import { MatPaginatorIntl, MatPaginatorModule } from '@angular/material/paginator';
import { MatSortHeaderIntl, MatSortModule } from '@angular/material/sort';
import { MatStepperIntl } from '@angular/material/stepper';
import { NgxMdModule } from 'ngx-md';

import { CoreModule } from '@datasus/core';
import { LayoutModule } from '@datasus/layout';

import { CommonsConfig } from './configs/commons.config';
import { PaginatorIntl } from './intl/paginator.intl';
import { StepperIntl } from './intl/stepper.intl';
import { SortHeaderIntl } from './intl/sort-header.intl';
import { DatepickerIntl } from './intl/datepicker.intl';
import { WelcomeComponent } from './components/welcome/welcome.component';
import { NotFoundComponent } from './components/not-found/not-found.component';
import { ErrorBackendComponent } from './components/error-backend/error-backend.component';
import { ForbiddenComponent } from './components/forbidden/forbidden.component';
import { ChooseProfileComponent } from './components/choose-profile/choose-profile.component';
import { FabExtendedComponent } from './components/fab-extended/fab-extended.component';
import { DynamicActionsComponent } from './components/dynamic-actions/dynamic-actions.component';
import { DynamicTableComponent } from './components/dynamic-table/dynamic-table.component';
import { ExpandableTableComponent } from './components/expandable-table/expandable-table.component';
import { ReleaseNotesComponent } from './components/release-notes/release-notes.component';
import { SafeHtmlPipe } from './pipes/safe-html.pipe';
import { SafeResourceUrlPipe } from './pipes/safe-resource-url.pipe';
import { SafeUrlPipe } from './pipes/safe-url.pipe';
import { SafeStylePipe } from './pipes/safe-style.pipe';
import { ConfigPipe } from './pipes/config.pipe';
import { UrlDownloadPipe } from './pipes/url-download.pipe';
import { UrlDownloadDeprecatedPipe } from './pipes/url-download-deprecated.pipe'; // TODO Remover depois
import { UrlDownloadPublicPipe } from './pipes/url-download-public.pipe'; // TODO Remover depois
import { FilterPipe } from './pipes/filter.pipe';
import { NormalizePipe } from './pipes/normalize.pipe';
import { DatePtAdapter } from './adapters/date-pt.adapter';
import { UrlDownloadAssinaturaPipe } from './pipes/url-download-assinatura.pipe';

const PORTUGUESE = 'pt';
const DATE_FORMATS = {
  parse: {
    dateInput: 'LL',
  },
  display: {
    dateInput: 'LL',
    monthYearLabel: 'MMM YYYY',
    dateA11yLabel: 'LL',
    monthYearA11yLabel: 'MMMM YYYY',
  },
};

registerLocaleData(localePt, PORTUGUESE, localePtExtra);

@NgModule({
  imports: [
    // Angular
    CommonModule,
    RouterModule,
    // Angular Flex-Layout
    FlexLayoutModule,
    // Angular Material
    MatButtonModule,
    MatCardModule,
    MatExpansionModule,
    MatIconModule,
    MatListModule,
    MatMenuModule,
    MatProgressBarModule,
    MatProgressSpinnerModule,
    MatPaginatorModule,
    MatSortModule,
    MatTableModule,
    MatTooltipModule,
    // Angular Markdown (NgxMd)
    NgxMdModule,
    // Datasus
    CoreModule,
    LayoutModule,
  ],
  declarations: [
    // Pipes
    UrlDownloadPipe,
    UrlDownloadDeprecatedPipe, // TODO Remover depois
    UrlDownloadPublicPipe, // TODO Remover depois
    ConfigPipe,
    SafeHtmlPipe,
    SafeResourceUrlPipe,
    SafeUrlPipe,
    SafeStylePipe,
    FilterPipe,
    NormalizePipe,
    UrlDownloadAssinaturaPipe,
    // Components
    ChooseProfileComponent,
    ErrorBackendComponent,
    ForbiddenComponent,
    NotFoundComponent,
    WelcomeComponent,
    FabExtendedComponent,
    DynamicActionsComponent,
    DynamicTableComponent,
    ExpandableTableComponent,
    ReleaseNotesComponent,
  ],
  exports: [
    // Pipes
    UrlDownloadPipe,
    UrlDownloadDeprecatedPipe, // TODO Remover depois
    UrlDownloadPublicPipe, // TODO Remover depois
    ConfigPipe,
    SafeHtmlPipe,
    SafeResourceUrlPipe,
    SafeUrlPipe,
    SafeStylePipe,
    FilterPipe,
    NormalizePipe,
    UrlDownloadAssinaturaPipe,
    // Components
    NotFoundComponent,
    FabExtendedComponent,
    DynamicActionsComponent,
    DynamicTableComponent,
    ExpandableTableComponent,
    ReleaseNotesComponent
  ],
  providers: [
    {
      provide: LOCALE_ID,
      useValue: PORTUGUESE,
    },
    {
      provide: MAT_DATE_LOCALE,
      useValue: PORTUGUESE,
    },
    {
      provide: DateAdapter,
      useClass: DatePtAdapter,
    },
    {
      provide: MAT_DATE_FORMATS,
      useValue: DATE_FORMATS,
    },
    {
      provide: MatPaginatorIntl,
      useClass: PaginatorIntl,
    },
    {
      provide: MatStepperIntl,
      useClass: StepperIntl,
    },
    {
      provide: MatSortHeaderIntl,
      useClass: SortHeaderIntl,
    },
    {
      provide: MatDatepickerIntl,
      useClass: DatepickerIntl,
    },
    {
      provide: 'CommonsConfig',
      useValue: {},
    },
  ]
})
export class CommonsModule {

  /**
   * Criando o Módulo com suas configurações
   * @param commonsConfig Configuração do módulo comum: Texto da home inicial, etc...
   */
  public static forRoot(commonsConfig: CommonsConfig): ModuleWithProviders<CommonsModule> {
    return {
      ngModule: CommonsModule,
      providers: [{
        provide: 'CommonsConfig',
        useValue: commonsConfig,
      }],
    };
  }
}
