import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, CanLoad, Router, Route, UrlSegment } from '@angular/router';
import { Observable, of as observableOf, throwError } from 'rxjs';
import { switchMap, take, catchError } from 'rxjs/operators';

import { DATASUS_ROUTE_FORBIDDEN } from '../constants/routes.contants';
import { PerfilModel } from '../models/perfil.model';
import { ErrorModel } from '../models/error.model';
import { SecurityService } from '../services/security.service';

@Injectable({
  providedIn: 'root'
})
export class SecurityGuard implements CanActivate, CanLoad {

  constructor(
    private router: Router,
    private securityService: SecurityService
  ) { }

  canActivate(
    next: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<boolean> | Promise<boolean> | boolean {
    try {
      console.log('[SecurityGuard::canActivate]');
      return this.checkRoles();
    } catch (error) {
      console.warn('[SecurityGuard::canActivate] catch error', error);
      return false;
    }
  }

  canLoad(
    route: Route,
    segments: UrlSegment[],
  ): Observable<boolean> | Promise<boolean> | boolean {
    try {
      console.log('[SecurityGuard::canLoad]');
      return this.checkRoles();
    } catch (error) {
      console.warn('[SecurityGuard::canLoad] catch error', error);
      return false;
    }
  }

  private checkRoles(): Observable<boolean> | boolean {
    if (this.securityService.hasPerfilSelecionado()) {
      console.log('[SecurityGuard::checkRoles] Já tem perfil selecionado');
      return true;
    }

    return this.securityService.requestProfiles().pipe(
      catchError((error: ErrorModel) => {
        console.warn('[SecurityGuard::checkRoles] Erro', error);
        this.router.navigate([`/${DATASUS_ROUTE_FORBIDDEN}`]);
        return throwError(false);
      }),
      switchMap((perfis: PerfilModel[]) => {
        this.securityService.setPerfis(perfis);
        if (!perfis || !perfis.length) {
          console.warn('[SecurityGuard::checkRoles] Sem perfis');
          this.router.navigate([`/${DATASUS_ROUTE_FORBIDDEN}`]);
          return observableOf(false);
        }
        if (perfis.length > 1) {
          console.log('[SecurityGuard::checkRoles] Mais de um perfil, vai para a tela de seleção');
          this.securityService.navigateToChooseProfile();
          return observableOf(false);
        }
        const perfil = perfis[0];
        if (!perfil.esferas || !perfil.esferas.length) {
          console.warn('[SecurityGuard::checkRoles] Perfil sem esferas', perfil);
          this.router.navigate([`/${DATASUS_ROUTE_FORBIDDEN}`]);
          return observableOf(false);
        }
        if (perfil.esferas.length > 1) {
          console.log('[SecurityGuard::checkRoles] Mais de uma esfera para o perfil, vai para a tela de seleção');
          this.securityService.navigateToChooseProfile();
          return observableOf(false);
        }
        const esfera = perfil.esferas[0];
        this.securityService.setPerfilSelecionado(perfil);
        this.securityService.setEsferaSelecionada(esfera);
        return observableOf(true);
      }),
    );
  }
}
