import { Injectable } from '@angular/core';
import { HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';

import { ErrorModel } from '../models/error.model';

@Injectable({
  providedIn: 'root'
})
export class ErrorInterceptor implements HttpInterceptor {

  intercept(
    request: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    return next.handle(request).pipe(
      catchError(this.handleHttpError.bind(this))
    );
  }

  private handleHttpError(response: HttpErrorResponse): Observable<never> {
    try {
      console.warn('[ErrorInterceptor::handleHttpError] message: "%s", status: %d', response.message, response.status);

      const okButHasSomeError = (error: any): string => {
        console.error('[ErrorInterceptor::okButHasSomeError] Algo errado na hora de montar o retorno.', error);
        console.warn('[ErrorInterceptor::okButHasSomeError] Verifique o tipo do retorno se bate com o que vem do backend.');
        return 'Erro ao montar o retorno.';
      };

      const badRequest = (error: any): string => {
        console.error('[ErrorInterceptor::badRequest] Erro negocial da API!', error);
        let messages = ['Erro de API'];
        if (error instanceof Array) {
          messages = error.map(
            (e: any) => e.message ?
              e.message : 'Erro de API: '.concat(e.error ? e.error : 'desconhecido')
          );
        }
        return messages.join('\n');
      };

      const unauthorized = (error: any): string => {
        console.error('[ErrorInterceptor::accessDennied] Sem autorização, sai fora do sistema!', error);
        return 'Sem autorização!';
      };

      const forbidden = (error: any): string => {
        console.error('[ErrorInterceptor::forbidden] Proibido!', error);
        return 'Você não tem permissão.';
      };

      const notFound = (error: any): string => {
        console.error('[ErrorInterceptor::notFound] Erro de não encontrado!', error);
        return 'Nenhum resultado encontrado.';
      };

      const methodNotAllowed = (error: any): string => {
        console.error('[ErrorInterceptor::methodNotAllowed] Método HTTP não existe!', error);
        console.warn('[ErrorInterceptor::methodNotAllowed] Verifique o método da requisição bate com o que está registrado no backend.');
        return 'Problema com a requisição.';
      };

      const internalServerError = (error: any): string => {
        console.error('[ErrorInterceptor::internalServerError] Erro interno no servidor!', error);
        // TODO tratamento temporário dessa mensagens que vem encapsuladas dentro de outra...
        const workaround = this.workaroundOnMessage((error.message || '') + '');
        return workaround.message || 'Erro interno no servidor!';
      };

      const requestFail = (error: any): string => {
        console.error('[ErrorInterceptor::serviceNotAvailable] Falhou o request!', error);
        return ':( Xii...! Serviço indisponível, tente novamente em alguns minutos.';
      };

      const statusPreprocessor = (originalStatus: number, originalError: any): number => {
        // TODO tratamento temporário dessa mensagens que vem encapsuladas dentro de outra...
        if (!originalError) {
          return originalStatus;
        }
        const workaround = this.workaroundOnMessage((originalError.message || '') + '');
        return workaround.status || originalStatus;
      };

      const errorMapping = {
        // 0×× Non state
        0: requestFail,

        // 1×× Informational

        // 100 Continue
        // 101 Switching Protocols
        // 102 Processing

        // 2×× Success
        200: okButHasSomeError,       // 200 OK
        // 201 Created
        // 202 Accepted
        // 203 Non-authoritative Information
        // 204 No Content
        // 205 Reset Content
        // 206 Partial Content
        // 207 Multi-Status
        // 208 Already Reported
        // 226 IM Used

        // 3×× Redirection
        // 300 Multiple Choices
        // 301 Moved Permanently
        // 302 Found
        // 303 See Other
        // 304 Not Modified
        // 305 Use Proxy
        // 307 Temporary Redirect
        // 308 Permanent Redirect

        // 4×× Client Error
        400: badRequest,              // 400 Bad Request
        401: unauthorized,            // 401 Unauthorized
        // 402 Payment Required
        403: forbidden,               // 403 Forbidden
        404: notFound,                // 404 Not Found
        405: methodNotAllowed,        // 405 Method Not Allowed
        // 406 Not Acceptable
        // 407 Proxy Authentication Required
        // 408 Request Timeout
        // 409 Conflict
        // 410 Gone
        // 411 Length Required
        // 412 Precondition Failed
        // 413 Payload Too Large
        // 414 Request-URI Too Long
        // 415 Unsupported Media Type
        // 416 Requested Range Not Satisfiable
        // 417 Expectation Failed
        // 418 I'm a teapot
        // 421 Misdirected Request
        // 422 Unprocessable Entity
        // 423 Locked
        // 424 Failed Dependency
        // 426 Upgrade Required
        // 428 Precondition Required
        // 429 Too Many Requests
        // 431 Request Header Fields Too Large
        // 444 Connection Closed Without Response
        // 451 Unavailable For Legal Reasons
        // 499 Client Closed Request

        // 5×× Server Error
        500: internalServerError,     // 500 Internal Server Error
        // 501 Not Implemented
        // 502 Bad Gateway
        // 503 Service Unavailable
        // 504 Gateway Timeout
        // 505 HTTP Version Not Supported
        // 506 Variant Also Negotiates
        // 507 Insufficient Storage
        // 508 Loop Detected
        // 510 Not Extended
        // 511 Network Authentication Required
        // 599 Network Connect Timeout Error
      };

      if (response.status in errorMapping) {
        return throwError(
          new ErrorModel(
            errorMapping[response.status](response.error),
            statusPreprocessor(response.status, response.error),
            response
          )
        );
      }
    } catch (e) {
      console.error(e, response);
    }

    return throwError(new ErrorModel(':( Ops! Algo inesperado aconteceu.', 0, response));
  }

  private workaroundOnMessage(message: string): { message: string, status: number } {
    const result = {
      message: null,
      status: null,
    };
    const str = message.replace(/\n+/g, '');
    const regex = /status\s(\d*).*message":"(.*)"\W/gmi;
    let matches: RegExpExecArray;
    while ((matches = regex.exec(str)) !== null) {
      if (matches.index === regex.lastIndex) {
          regex.lastIndex++;
      }
      matches.forEach((match: string, groupIndex: number) => {
        if (groupIndex === 1) {
          result.status = match;
        }
        if (groupIndex === 2) {
          result.message = match;
        }
      });
    }

    return result;
  }
}
