import { Location } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable, of as observableOf, throwError, Subject } from 'rxjs';
import { catchError, map, switchMap, take, tap } from 'rxjs/operators';

import { ErrorModel } from '../models/error.model';
import { UsuarioModel } from '../models/usuario.model';
import { redirectTo } from '../utils/functions';
import { urlEncode } from '../utils/encoder-decoder';
import { ConfigService } from './config.service';
import { SecurityService } from './security.service';


@Injectable({
  providedIn: 'root',
})
export class AuthService {

  user: UsuarioModel;

  private gettingOut$: Subject<string> = new Subject<string>();

  constructor(
    private location: Location,
    private httpClient: HttpClient,
    private configService: ConfigService,
    private securityService: SecurityService,
  ) { }

  /**
   * Retorna o Observable com a URL de saida do sistema
   */
  gettingOutEvent(): Observable<string> {
    return this.gettingOut$.asObservable();
  }

  /**
   * Retorna o Observable se está logado ou não
   */
  isAuthenticated(): Observable<boolean> {
    return this.configService.get('backendUrl').pipe(
      switchMap((backendUrl: string) => {
        console.log('[AuthService::isAuthenticated] Vamos verificar se já está autenticado', backendUrl);
        const request$ = this.httpClient.get<boolean>(`${backendUrl}/oauth/verify`).pipe(
          take(1),
          catchError((error: ErrorModel) => {
            if (error.code === 401) {
              console.warn('[AuthService::isAuthenticated] Não está autenticado');
            } else if (error.code === 403) {
              console.warn('[AuthService::isAuthenticated] Acesso proíbido cadastrado');
            } else {
              console.warn('[AuthService::isAuthenticated] Ops, erro na verificação', error);
            }
            return throwError(error);
          }),
          map((result: boolean) => {
            console.log('[AuthService::isAuthenticated] Acessou a URL do backend de boa!', result);
            if (!result) {
              throw result;
            }
            return result;
          }),
        );
        return request$;
      }),
    );
  }

  /**
   * Retorna o nome completo do usuário autorizado
   */
  getUser(): Observable<UsuarioModel> {
    console.log('[AuthService::getUser] Vamos buscar os dados do usuário ...');
    if (this.user) {
      console.log('[AuthService::getUser] ... Pegou do cache!');
      return observableOf(this.user);
    }

    console.log('[AuthService::getUser] ... Vai pegar o backend ...');
    return this.isAuthenticated().pipe(
      switchMap((isAutenticated: boolean) => {
        if (!isAutenticated) {
          console.log('[AuthService::getUser] ... Não está autenticado!');
          return throwError(null);
        }
        console.log('[AuthService::getUser] ... Está autenticado, pode pegar lá!');
        return this.configService.get('backendUrl').pipe(
          switchMap((backendUrl: string) => this.httpClient.get<UsuarioModel>(`${backendUrl}/acesso/usuario`).pipe(
            tap((usuario: UsuarioModel) => this.user = usuario)
          ))
        );
      })
    );
  }

  /**
   * Efetua o logout do sistema
   */
  logout(): void {
    this.configService.get('backendUrl').subscribe(
      (backendUrl: string) => {
        const logoutUrl = `${backendUrl}/logout`;
        console.log('[AuthService::logout] Redirecionando para deslogar:', logoutUrl);
        this.getOut(logoutUrl);
      }
    );
  }


  login(): void {
    this.configService.getList([
      'backendUrl',
      'frontendUrl',
    ]).subscribe((configs: string[]) => {
      const backendUrl = configs[0];
      const returnPath = this.location.path(true) || '';
      const frontendUrl = urlEncode(`${configs[1]}${returnPath}`);
      const autorizeUrl = `${backendUrl}/oauth/authorize?redirect_uri=${frontendUrl}`;
      console.log('[AuthService::redirectToAuthorize] Redirecionando para autorizar:', autorizeUrl);
      this.getOut(autorizeUrl);
    });
  }

  getOut(url: string): void {
    this.gettingOut$.next(url);
    this.user = null;
    this.securityService.clear();
    setTimeout(() => redirectTo(url), 1000);
  }

  cookie(): Observable<string> {
    return this.configService.get('backendUrl').pipe(
      switchMap(
        (backendUrl: string) => this.httpClient.get(`${backendUrl}/cookie`, {
          responseType: 'text'
        }).pipe(take(1))
      )
    );
  }
}
