import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, of as observableOf, forkJoin } from 'rxjs';
import { tap, take, switchMap } from 'rxjs/operators';

import { ConfigService } from './config.service';
import { DominioModel } from '../models/dominio.model';

/**
 * Serviço responsável por trazer os dominios
 * de `/dominio/`, `/dominio/itens` ou `/dominio/{nome-do-dominio}`
 */
@Injectable({
  providedIn: 'root'
})
export class DominioService {

  /**
   * Cache dos dominios
   */
  private domainsCache: DominioModel[] = [];
  private domainsItensCache: string[] = [];
  private domainCache = new Map<string, DominioModel>();

  /**
   * Inicializador da classe
   * @param httpClient Cliente para as requisiçõe HTTP
   * @param configService Serviço responsável por trazer as configurações do sistema
   */
  constructor(
    private httpClient: HttpClient,
    private configService: ConfigService
  ) { }

  /**
   * Retorna um `Observable`. Se assinado vai
   * trazer todas os dominios em
   * um objeto só
   */
  getAll(): Observable<DominioModel[]> {
    if (this.domainsCache.length > 0) {
      return observableOf(this.domainsCache);
    } else {
      return this.configService.get('backendUrl').pipe(
        switchMap(
          (backendUrl: string) => this.httpClient.get<DominioModel[]>(`${backendUrl}/dominio/todos`).pipe(
            take(1),
            tap((domainList: DominioModel[]) => this.domainsCache = domainList),
          )
        )
      );
    }
  }

  /**
   * Retorna um `Observable`. Se assinado vai
   * trazer o dominio selecionada
   * @param domainName Nome do dominio a ser retornado no `Observable`
   */
  get(domainName: string): Observable<DominioModel> {
    if (this.domainCache.has(domainName)) {
      return observableOf(this.domainCache.get(domainName));
    } else {
      return this.configService.get('backendUrl').pipe(
        switchMap(
          (backendUrl: string) => this.httpClient.get<DominioModel>(`${backendUrl}/dominio/${domainName}`).pipe(
            take(1),
            tap((domain: DominioModel) => this.domainCache.set(domainName, domain)),
          )
        )
      );
    }
  }

  /**
   * Retorna um `Observable`. Se assinado vai
   * trazer os dominios selecionadas
   * em uma lista
   * @param domainNames Lista de nome dos dominios a serem retornados no `Observable`
   */
  getList(domainNames: string[]): Observable<DominioModel[]> {
    return forkJoin(
      domainNames.map((domainName: string) => this.get(domainName))
    ).pipe(take(1));
  }

  /**
   * Retorna um `Observable`. Se assinado vai
   * trazer os nomes de todos os dominios
   * em uma lista
   */
  getItemNames(): Observable<string[]> {
    if (this.domainsItensCache.length > 0) {
      return observableOf(this.domainsItensCache);
    } else {
      return this.configService.get('backendUrl').pipe(
        switchMap(
          (backendUrl: string) => this.httpClient.get<string[]>(`${backendUrl}/dominio/`).pipe(
            take(1),
            tap((domainNames: string[]) => this.domainsItensCache = domainNames),
          )
        )
      );
    }
  }
}
