import { Injectable } from '@angular/core';
import { Location } from '@angular/common';
import { Router } from '@angular/router';
import { Observable, BehaviorSubject } from 'rxjs';

import { ConfigService } from './config.service';
import { LocalStorage } from '../utils/local.storage';
import { PerfilModel } from '../models/perfil.model';
import { EsferaModel } from '../models/esfera.model';
import {
  STORAGE_KEY_PERFIS,
  STORAGE_KEY_PERFIL_SELECIONADO,
  STORAGE_KEY_ESFERA_SELECIONADA,
} from '../constants/security.constants';
import {
  DATASUS_ROUTE_CHOOSE_PROFILE,
  DATASUS_ROUTE_FORBIDDEN,
  DATASUS_ROUTE_ERROR_BACKEND,
  DATASUS_ROUTE_HOME,
} from '../constants/routes.contants';
import { HttpClient } from '@angular/common/http';
import { take, switchMap, tap } from 'rxjs/operators';

/**
 * Serviço que tem os atalhos para acessar
 * informações como perfil e esfera
 */
@Injectable({
  providedIn: 'root'
})
export class SecurityService {

  private perfis$: BehaviorSubject<PerfilModel[]> = new BehaviorSubject([]);
  private perfilSelecionado$: BehaviorSubject<PerfilModel> = new BehaviorSubject(null);
  private esferaSelecionada$: BehaviorSubject<EsferaModel> = new BehaviorSubject(null);

  constructor(
    private location: Location,
    private router: Router,
    private httpClient: HttpClient,
    private configService: ConfigService,
  ) { }

  requestProfiles(): Observable<PerfilModel[]> {
    return this.configService.getList([
      'backendUrl',
      'clientId',
    ]).pipe(
      switchMap((configs: string[]) => {
        const backendUrl = configs[0];
        const clientId = configs[1];
        console.log('[SecurityService::requestProfiles] Vamos recuperar os perfis', backendUrl, clientId);
        return this.httpClient.get<PerfilModel[]>(
          `${backendUrl}/acesso/sistema/${clientId}/perfil`
        ).pipe(
          take(1),
          tap((profiles: PerfilModel[]) => {
            console.log('[SecurityService::requestProfiles] Perfis e suas esferas', profiles);
            this.setPerfis(profiles);
          })
        );
      }),
    );
  }

  navigateToChooseProfile(forceRedirectToHome: boolean = false): void {
    const returnPath = forceRedirectToHome
      ? DATASUS_ROUTE_HOME
      : (this.location.path(true) || DATASUS_ROUTE_HOME);
    const route: (string | {[returnPath: string]: string})[] = [`/${DATASUS_ROUTE_CHOOSE_PROFILE}`];

    if (!!returnPath) {
      const ignoreRoutes: string[] = [
        DATASUS_ROUTE_CHOOSE_PROFILE,
        DATASUS_ROUTE_FORBIDDEN,
        DATASUS_ROUTE_ERROR_BACKEND,
      ];
      const ignoreRoutesFiltered = ignoreRoutes.filter((ignoreRoute: string) => !returnPath.includes(ignoreRoute));
      if (ignoreRoutes.length === ignoreRoutesFiltered.length) {
        route.push({ returnPath });
      }
    }

    console.log('[SecurityService::navigateToChooseProfile]', route);
    this.router.navigate(route);
  }

  chooseProfileAndNavigate({ returnPath, perfil, esfera }: { returnPath?: string; perfil: PerfilModel; esfera?: EsferaModel; }): void {
    console.log('[SecurityService::chooseProfileAndNavigate]', perfil, esfera);
    this.setPerfilSelecionado(perfil);
    if (esfera) {
      this.setEsferaSelecionada(esfera);
    } else {
      this.clearEsferaSelecionada();
    }
    console.log('[SecurityService::chooseProfileAndNavigate] go to %s', '/' + (returnPath || ''));
    this.router.navigate(['/' + (returnPath || '')]);
  }

  /**
   * Limpa os dados de perfil e esfera
   */
  clear(): void {
    this.clearPerfis();
    this.clearPerfilSelecionado();
    this.clearEsferaSelecionada();
  }
  /**
   * Limpa a lista de perfis
   */
  clearPerfis(): void {
    LocalStorage.removeItem(this.key(STORAGE_KEY_PERFIS));
    this.perfis$.next([]);
  }
  /**
   * Verifica se tem lista de perfis
   */
  hasPerfis(): boolean {
    return !!(LocalStorage.getItem(this.key(STORAGE_KEY_PERFIS)) || []).length;
  }
  /**
   * Recupera a lista de perfis
   */
  getPerfis(): Observable<PerfilModel[]> {
    this.perfis$.next(LocalStorage.getItem(this.key(STORAGE_KEY_PERFIS)) || []);
    return this.perfis$.asObservable();
  }
  /**
   * Salva a lista de perfis
   * @param perfis Lista de perfis
   */
  setPerfis(perfis: PerfilModel[]): void {
    LocalStorage.setItem(this.key(STORAGE_KEY_PERFIS), this.decoratePerfis(perfis));
    this.perfis$.next(perfis);
  }
  /**
   * Limpa o perfil selecionado
   */
  clearPerfilSelecionado(): void {
    LocalStorage.removeItem(this.key(STORAGE_KEY_PERFIL_SELECIONADO));
    this.perfilSelecionado$.next(null);
    this.clearEsferaSelecionada();
  }
  /**
   * Verifica se tem perfil selecionado
   */
  hasPerfilSelecionado(): boolean {
    return this.hasDescricao(this.getStoredPerfilSelecionado());
  }
  /**
   * Recupera o observable do perfil selecionado
   */
  getPerfilSelecionado(): Observable<PerfilModel> {
    this.perfilSelecionado$.next(this.getStoredPerfilSelecionado());
    return this.perfilSelecionado$.asObservable();
  }
  /**
   * Recupera o perfil selecionado armazenada
   */
  getStoredPerfilSelecionado(): PerfilModel {
    return LocalStorage.getItem(this.key(STORAGE_KEY_PERFIL_SELECIONADO));
  }
  /**
   * Salva o perfil selecionado
   * @param perfil Perfil selecionado
   */
  setPerfilSelecionado(perfil: PerfilModel): void {
    LocalStorage.setItem(this.key(STORAGE_KEY_PERFIL_SELECIONADO), this.decoratePerfil(perfil));
    this.perfilSelecionado$.next(perfil);
    this.clearEsferaSelecionada();
  }
  /**
   * Limpa a esfera selecionada
   */
  clearEsferaSelecionada(): void {
    LocalStorage.removeItem(this.key(STORAGE_KEY_ESFERA_SELECIONADA));
    this.esferaSelecionada$.next(null);
  }
  /**
   * Verifica se tem esfera selecionada
   */
  hasEsferaSelecionada(): boolean {
    return this.hasDescricao(this.getStoredEsferaSelecionada());
  }
  /**
   * Recupera o observable da esfera selecionada
   */
  getEsferaSelecionada(): Observable<EsferaModel> {
    this.esferaSelecionada$.next(this.getStoredEsferaSelecionada());
    return this.esferaSelecionada$.asObservable();
  }
  /**
   * Recupera a esfera selecionada armazenada
   */
  getStoredEsferaSelecionada(): EsferaModel {
    return LocalStorage.getItem(this.key(STORAGE_KEY_ESFERA_SELECIONADA));
  }
  /**
   * Salva a esfera selecionada
   * @param esfera Esfera selecionada
   */
  setEsferaSelecionada(esfera: EsferaModel): void {
    LocalStorage.setItem(this.key(STORAGE_KEY_ESFERA_SELECIONADA), this.decorateEsfera(esfera));
    this.esferaSelecionada$.next(esfera);
  }

  private decoratePerfis(perfis: PerfilModel[]): PerfilModel[] {
    const perfisWithDecoratedEsferas = (perfis || []).map((perfil: PerfilModel) => this.decoratePerfil(perfil));
    return perfisWithDecoratedEsferas;
  }

  private decoratePerfil(perfil: PerfilModel): PerfilModel {
    const perfilWithDecoratedEsferas = { ...perfil };
    perfilWithDecoratedEsferas.esferas = this.decorateEsferas(perfilWithDecoratedEsferas.esferas);
    return perfilWithDecoratedEsferas;
  }

  private decorateEsferas(esferas: EsferaModel[]): EsferaModel[] {
    const esferasDecorated = (esferas || []).map((esfera: EsferaModel) => this.decorateEsfera(esfera));
    return esferasDecorated;
  }

  private decorateEsfera(esfera: EsferaModel): EsferaModel {
    const esferaDecorated = { ...esfera };
    esferaDecorated.esferaAtuacaoList = this.extractList(esfera.esferaAtuacao);
    esferaDecorated.descricaoEsferaAtuacaoList = this.extractList(esfera.descricaoEsferaAtuacao);
    return esferaDecorated;
  }

  private extractList(source: string = '') {
    return (source || '').replace(/\/$/, '').split('/').filter((item: string) => !!item);
  }

  private hasDescricao(item: {descricao: string}): boolean {
    return (!!item && !!item.descricao);
  }

  private key(key: string): string {
    return `${key} @ ${this.configService.appKey}`;
  }
}
