import { Component, OnInit, ViewChild } from '@angular/core';
import { FormControl } from '@angular/forms';
import { MatStepper } from '@angular/material/stepper';
import { MatDialogRef } from '@angular/material/dialog';
import { finalize } from 'rxjs/operators';
import { AliveResult } from '../../models/alive-result.model';
import { EvoSdkService } from '../../services/evo-sdk.service';
import { IsTokenPresentResult } from '../../models/is-token-present-result.model';
import { GetTokenInfoResult } from '../../models/get-token-info-result.model';
import { SignAndSendResult } from '../../models/sign-and-send-result.model';



@Component({
  selector: 'datasus-assinatura-digital-test-dialog',
  templateUrl: './assinatura-digital-test-dialog.component.html',
  styleUrls: ['./assinatura-digital-test-dialog.component.scss']
})
export class AssinaturaDigitalTestDialogComponent implements OnInit {

  @ViewChild('stepper')
  stepper: MatStepper;

  loading = false;

  aliveCtrl = new FormControl();

  aliveMessage: string;
  isTokenPresentMessage: string;
  getTokenInfoList: string[];
  signAndSendMessage: string;

  constructor(
    private matDialogRef: MatDialogRef<AssinaturaDigitalTestDialogComponent>,
    private evoSdkService: EvoSdkService,
  ) { }

  ngOnInit(): void {
    this.aliveCtrl.setErrors({alive: true});
    this.aliveMessage = '';
    this.isTokenPresentMessage = '';
    this.getTokenInfoList = [];
  }

  aliveTest(): void {
    this.aliveCtrl.setErrors({alive: true});
    this.aliveMessage = '';
    this.loading = true;
    this.evoSdkService.alive().pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (result: AliveResult) => {
        if (result.alive) {
          this.aliveCtrl.setErrors(null);
          this.stepper.next();
        } else {
          this.aliveMessage = `SDK-Desktop não está ativo! Versão: ${result.version}.`;
          console.error(result);
        }
      },
      (error: any) => {
        this.aliveMessage = 'Erro ao processar a serviço "alive" do SDK-Desktop.';
        console.error(error);
      },
    );
  }

  isTokenPresentTest(): void {
    this.isTokenPresentMessage = '';
    this.loading = true;
    this.evoSdkService.isTokenPresent().pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (result: IsTokenPresentResult) => {
        if (result.isTokenPresent) {
          this.stepper.next();
        } else if (result.isPkcs11) {
          this.isTokenPresentMessage = 'SDK-Desktop não está com token configurado, porém existe um PKCS11.';
          this.stepper.next();
        } else {
          this.isTokenPresentMessage = 'SDK-Desktop não está com token configurado.';
          console.error(result);
        }
      },
      (error: any) => {
        this.isTokenPresentMessage = 'Erro ao processar a serviço "isTokenPresent" do SDK-Desktop.';
        console.error(error);
      },
    );
  }

  getTokenInfoTest(): void {
    this.getTokenInfoList = [];
    this.loading = true;
    this.evoSdkService.getTokenInfo().pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (result: GetTokenInfoResult) => {
        this.getTokenInfoList.push(`ATR: ${result.atr}`);
        if (!!result.tokenManufacturerID) {
          this.getTokenInfoList.push(`ID do Fabricante: ${result.tokenManufacturerID}`);
        }
        if (!!result.tokenModel) {
          this.getTokenInfoList.push(`Modelo: ${result.tokenModel}`);
        }
        if (!!result.tokenSerialNumber) {
          this.getTokenInfoList.push(`Nº Serial: ${result.tokenSerialNumber}`);
        }
      },
      (error: any) => {
        this.getTokenInfoList.push('Erro ao processar a serviço "getTokenInfo" do SDK-Desktop.');
        console.error(error);
      },
    );
  }

  signAndSendTest(): void {
    const idArquivo = 'exemplo-20200102-ce7efd4c-a60d-43d1-9502-175ff69ce6b4';

    this.signAndSendMessage = '';
    this.loading = true;
    this.evoSdkService.signAndSend(idArquivo, {
      'Testado em': new Date().toISOString(),
      'Golpe do Ryu': 'Hadouken!',
    }).pipe(
      finalize(() => this.loading = false)
    ).subscribe(
      (result: SignAndSendResult) => {

        if (!!result.failure && !!result.failure.causes) {
          let failMessages: string;
          result.failure.causes.forEach(cause => {
            failMessages = failMessages + cause.exception + ':' + cause.message;
          });
          this.signAndSendMessage = 'Erro ao assinar o arquivo. Favor verificar no console.';
          console.log(result);
        } else {
          this.signAndSendMessage = 'Arquivo assinado com sucesso.';
        }

      },
      (error: any) => {
        this.signAndSendMessage = 'Erro ao processar a serviço "signAndSend" do SDK-Desktop.';
        console.error(error);
      },
    );
  }

  config(): void {
    this.loading = true;
    this.evoSdkService.config().pipe(
      finalize(() => {
        this.loading = false;
        this.signAndSendMessage = '';
        this.isTokenPresentMessage = '';
      })
    ).subscribe(
      () => console.log('Fechou as configurações'),
      (error: any) => console.error(error),
    );
  }

  close(): void {
    this.matDialogRef.close();
  }

}
