import { Pipe, PipeTransform } from '@angular/core';

@Pipe({
  name: 'datasusRelativeDate'
})
export class RelativeDatePipe implements PipeTransform {

  private readonly MILLISECOND = 1;
  private readonly SECOND = this.MILLISECOND * 1000;
  private readonly MINUTE = this.SECOND * 60;
  private readonly HOUR = this.MINUTE * 60;
  private readonly DAY = this.HOUR * 24;
  private readonly WEEK = this.DAY * 7;

  /**
   * Retorna um texto com a data relativa
   *
   * Quando passado o parametro {lowAccuracy} os possíveis retornos são:
   *   Hoje,
   *   Ontem,
   *   Essa semana,
   *   Semana passada,
   *   Mês passado,
   *   Mais antigas
   * No contrário:
   *   Agora pouco,
   *   Minutos atrás,
   *   Horas atrás,
   *   Dias atrás,
   *   Semanas atrás,
   *   Meses atrás
   *   Anos atrás
   */
  transform(
    value: (number | string | Date),
    valueToCompare: (number | string | Date),
    lowAccuracy: boolean = false
  ): string {
    const dateToCompare = new Date(valueToCompare || Date.now());
    const date = new Date(value);
    const diff = (+dateToCompare - +date);
    const diffMonth = this.getFullYearMonth(dateToCompare) - this.getFullYearMonth(date);

    if (0 > diff) {
      return 'Data futura';
    }

    if (lowAccuracy === true) {
      return this.lowAccurancyCalc(date, dateToCompare, diff, diffMonth);
    }

    return this.highAccurancyCalc(date, dateToCompare, diff, diffMonth);
  }

  private getFullYearMonth(date: Date): number {
    return +(
      date.getFullYear()
        .toString()
        .concat((date.getMonth() < 9 ? '0' : ''))
        .concat((date.getMonth() + 1).toString())
    );
  }

  private highAccurancyCalc(date: Date, dateToCompare: Date, diff: number, diffMonth: number): string {

    if (diff <= this.MINUTE) {
      return 'Agora pouco';
    }

    if (diff <= this.HOUR) {
      return 'Minutos atrás';
    }

    if (diff <= this.DAY) {
      return 'Horas atrás';
    }

    if (diff <= this.WEEK) {
      return 'Dias atrás';
    }

    if (diff > this.WEEK && diffMonth <= 1) {
      return 'Semanas atrás';
    }

    if (diffMonth <= 12 || diffMonth <= 100 ) {
      return 'Meses atrás';
    }

    return 'Anos atrás';
  }


  private lowAccurancyCalc(date: Date, dateToCompare: Date, diff: number, diffMonth: number): string {
    if (diff < this.DAY && date.getDate() === dateToCompare.getDate()) {
      return 'Hoje';
    }

    if (diff < (this.DAY * 2) && (date.getDate() + 1) === dateToCompare.getDate()) {
      return 'Ontem';
    }

    const dayOfWeek = date.getDay();
    const dateToCompareOfWeek = dateToCompare.getDay();

    if (diff < this.WEEK && dayOfWeek <= (dateToCompareOfWeek - 2)) {
      return 'Essa semana';
    }

    if ((diff - (dateToCompareOfWeek * this.DAY))  <= this.WEEK) {
      return 'Semana passada';
    }

    if (diffMonth === 1) {
      return 'Mês passado';
    }

    return 'Mais antiga';
  }
}
