import { Inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, of as observableOf } from 'rxjs';
import { switchMap, tap, delay, map } from 'rxjs/operators';

import { SecurityService, PerfilModel, ConfigService, EsferaModel } from '@datasus/core';

import { MenuModel } from '../models/menu.model';
import { LayoutConfig } from '../configs/layout.config';

@Injectable({
  providedIn: 'root'
})
export class MenuService {

  private cache: Map<string, MenuModel[]> = new Map();
  private perfilSelecionado: PerfilModel;

  constructor(
    @Inject('LayoutConfig')
    private layoutConfig: LayoutConfig,
    private httpClient: HttpClient,
    private configService: ConfigService,
    private securityService: SecurityService,
  ) { }

  filterItem: (menuItem: MenuModel) => boolean;

  reset(): void {
    this.cache.clear();
  }

  get(): Observable<MenuModel[]> {
      return this.securityService.getPerfilSelecionado().pipe(
        switchMap((perfil: PerfilModel) => {
          this.perfilSelecionado = perfil;
          return this.securityService.getEsferaSelecionada();
        }),
        switchMap(this.perfilSwitchToMenu.bind(this)),
        delay(0),
      );
  }

  private perfilSwitchToMenu(esfera?: EsferaModel): Observable<MenuModel[]> {
    const perfil = this.perfilSelecionado;

    if (!perfil) {
      if (this.layoutConfig.localMenu) {
        return observableOf(this.layoutConfig.localMenu);
      }
      return observableOf([]);
    }

    const key = perfil.sigla
      + (!!esfera ? (esfera.id || '').toString() + (esfera.esferaAtuacao || '') : '');

    if (this.cache.has(key)) {
      console.log('Menu do cache:', key);
      return observableOf(this.cache.get(key));
    }

    return this.configService.getList([
      'backendUrl',
      'clientId',
    ]).pipe(
      switchMap(
        (configs: string[]) => {
          const backendUrl = configs[0];
          const clientId = configs[1];
          console.log('Menu: ', key, backendUrl, clientId);
          const menu$ = this.httpClient.get<MenuModel[]>(
            `${backendUrl}/acesso/sistema/${clientId}/perfil/${perfil.sigla}/menu`
          );
         return menu$;
        }
      ),
      switchMap(
        (menuFinal: MenuModel[]) => {
          if (this.layoutConfig.localMenu) {
            this.layoutConfig.localMenu.forEach((menuLocal: MenuModel) => menuFinal.push(menuLocal));
          }
          console.log('Menu composto: ', menuFinal);
          return observableOf(menuFinal);
        }
      ),
      map(
        (menu: MenuModel[]) => menu.filter(
          (menuItem: MenuModel) => {
            if (typeof this.filterItem === 'function') {
              const filtered = this.filterItem(menuItem);
              console.log('Filter Menu: ', menuItem, filtered);
              return filtered;
            }
            return true;
          }
        )
      ),
      tap(
        (menu: MenuModel[]) => this.cache.set(key, menu)
      ),
    );
  }
}
