package br.gov.datasus.scodes.controller;

import java.util.List;

import javax.validation.Valid;

import br.gov.datasus.scodes.model.dto.TipoVaraDTO;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.CidadeDTO;
import br.gov.datasus.scodes.model.dto.VaraDTO;
import br.gov.datasus.scodes.model.dto.filter.VaraFilterDTO;
import br.gov.datasus.scodes.service.VaraService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

@Api(tags = "Vara")
@RequestMapping(value = "/vara")
@RequiredArgsConstructor
@RestController
public class VaraController {

    private final VaraService service;

    @ApiOperation(value = "Obter vara por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<VaraDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Pesquisa de varas paginada")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<VaraDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<VaraFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.pesquisarPaginado(pageableModel));
    }

    @ApiOperation(value = "Incluir vara")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<VaraDTO> incluir(@Valid @RequestBody VaraDTO dto) {
        dto = service.incluir(dto);
        return new ResponseEntity<VaraDTO>(dto, HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar vara")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<VaraDTO> alterar(@Valid @RequestBody VaraDTO dto, @PathVariable(name = "id") Long id) {
        dto = service.alterar(id, dto);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "Excluir vara por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }

    @ApiOperation(value = "Pesquisar cidade da vara")
    @PostMapping(value = "/pesquisar-cidade-vara/uf{uf}/cidade/{nome}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<CidadeDTO>> pesquisarMunicipioVara(@Valid @PathVariable(name = "uf") String uf, @Valid @PathVariable(name = "nome") String nome) {
        return ResponseEntity.ok(service.pesquisarCidadeVara(uf, nome));
    }

    @ApiOperation(value = "Obter municipios da vara por nome")
    @GetMapping(value = "/nome-municipio/{nome}/uf-municipio/{uf}",produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<CidadeDTO>> obterMunicipiosDaVaraPorNome(@PathVariable(name = "nome") String nome, @PathVariable(name = "uf") String uf) {
        List<CidadeDTO> lista = service.obterMunicipiosDaVaraPorNome(nome, uf);
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter tipos da vara e vara por nome e codigo municipio")
    @GetMapping(value = "/nome-municipio/{nome}/codigo-municipio/{cidadeId}",produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<VaraDTO>> pesquisarTipoVaraEVaraPorNomeCidade(@PathVariable(name = "nome") String nome, @PathVariable(name = "cidadeId") Long cidadeId) {
        List<VaraDTO> lista = service.pesquisarTipoVaraEVaraPorNomeCidade(cidadeId, nome);
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter tipos da vara por nome e codigo municipio")
    @GetMapping(value = "/tipo-vara/codigo-municipio/{cidadeId}",produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<TipoVaraDTO>> obterTiposVaraPorCodigoMunicipio(@PathVariable(name = "cidadeId") Long cidadeId) {
        List<TipoVaraDTO> lista = service.obterTiposVaraPorCodigoMunicipio(cidadeId);
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter varas por codigo tipo vara")
    @GetMapping(value = "/codigo-tipo-vara/{tipoVaraId}",produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<VaraDTO>> obterVarasPorCodigoTipoVara(@PathVariable(name = "tipoVaraId") Long tipoVaraId) {
        List<VaraDTO> lista = service.obterVarasPorCodigoTipoVara(tipoVaraId);
        return ResponseEntity.ok(lista);
    }
}

