package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.constants.AlteracaoDataRetornoConstants;
import br.gov.datasus.scodes.model.dto.*;
import br.gov.datasus.scodes.model.dto.filter.*;
import br.gov.datasus.scodes.model.dto.projection.ResultadoAlterarDataRetornoDTO;
import br.gov.datasus.scodes.model.entity.AlteracaoDataRetorno;
import br.gov.datasus.scodes.model.mapper.AlteracaoDataRetornoMapper;
import br.gov.datasus.scodes.model.mapper.RelatorioDataRetornoDemandaMapper;
import br.gov.datasus.scodes.model.mapper.ResultadoPesquisaDRAutorDemandaReciboMapper;
import br.gov.datasus.scodes.model.mapper.ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoMapper;
import br.gov.datasus.scodes.report.builder.AlteracaoDataRetornoReportBuilder;
import br.gov.datasus.scodes.repository.AlteracaoDataRetornoRepository;
import br.gov.datasus.scodes.repository.AutorDemandaRepository;
import br.gov.datasus.scodes.validation.AlteracaoDataRetornoServiceValidation;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static br.gov.datasus.scodes.report.builder.util.ConstantsJasper.REPORT_RELATORIO_DATA_RETORNO;

@Log4j2
@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class AlteracaoDataRetornoService {

    private static final String SOURCE_PESQUISA_DR_AD_RECIBO = "pesquisa_data_retorno_autor_demanda_recibo";
    private static final String SOURCE_PESQUISA_DR_AD_RECIBO_ORDEM_PRODUTO = "pesquisa_data_retorno_autor_demanda_recibo_ordem_produto";
    private static final String SOURCE_PESQUISA_DR_AD_ALTERACAO_DT_RETORNO = "pesquisa_data_retorno_autor_demanda_alteracao_data_retorno";
    private static final String SOURCE_RELATORIO_DR_AUTOR_DEMANDA = "relatorio_data_retorno_autor_demanda";

    private final AutorDemandaRepository autorDemandaRepository;
    private final AlteracaoDataRetornoRepository repository;
    private final AlteracaoDataRetornoMapper mapper;
    private final AlteracaoDataRetornoProdutoService alteracaoDataRetornoProdutoService;
    private final ReciboOrdemProdutoService reciboOrdemProdutoService;

    private final ResultadoPesquisaDRAutorDemandaReciboMapper resultadoPesquisaDRAutorDemandaReciboMapper;
    private final ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoMapper  resultadoPesquisaDRAutorDemandaReciboOrdemProdutoMapper;
    private final RelatorioDataRetornoDemandaMapper relatorioDataRetornoDemandaMapper;

    private final AlteracaoDataRetornoServiceValidation serviceValidation;
    private final AlteracaoDataRetornoReportBuilder alteracaoDataRetornoReportBuilder;

    public AlteracaoDataRetornoDTO obterPorId(Long id) {
        AlteracaoDataRetorno entity = obterEntityPorId(id);
        return mapper.toDto(entity);
    }

    public List<AlteracaoDataRetornoDTO> obterTodos() {
        Iterable<AlteracaoDataRetorno> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public List<ResultadoAlterarDataRetornoDTO> obterResultadoAlterarDataRetorno(Long codigoAlteracaoDataRetorno) {
        return repository.obterResultadoAlterarDataRetorno(codigoAlteracaoDataRetorno);
    }

    public PageableResponseModel<AlteracaoDataRetornoDTO> obterTodosPaginado(PageableRequestModel<AlteracaoDataRetornoFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "alteracaoDataRetorno");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            parametersAppendFilter(parameters, filter.getIdStatus(), "idStatus");
            parametersAppendLikeFilter(parameters, filter.getTitulo(), "titulo");
            parametersAppendFilter(parameters, filter.getDataInicio(), "dataInicio");
            parametersAppendFilter(parameters, filter.getDataFim(), "dataFim");
        });

        PageableResponseModel<AlteracaoDataRetorno> pageableResponseModel = repository
            .findAll(AlteracaoDataRetornoConstants.QUERYNAME.OBTER_TODOS_PAGINADO, parameters, pagination, AlteracaoDataRetorno.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public PageableResponseModel<ResultadoPesquisaDRAutorDemandaReciboDTO> pesquisaDTAutorDemandaRecibo(PageableRequestModel<PesquisaDRAutorDemandaReciboFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel);
        DynamicParameters parameters = getParametersFilterDTO(pageableModel);
        return autorDemandaRepository.findAllNative(SOURCE_PESQUISA_DR_AD_RECIBO, parameters, pagination, resultadoPesquisaDRAutorDemandaReciboMapper);
    }

    public List<ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTO> pesquisaDRAutorDemandaReciboOrdemProduto(PesquisaDRAutorDemandaReciboOrdemProdutoFilterDTO dtoFilter) {
        DynamicParameters parameters = getParametersPesquisaDataRetornoAutorReciboOrdemProdutoFilter(dtoFilter);
        return autorDemandaRepository.findAllNative(SOURCE_PESQUISA_DR_AD_RECIBO_ORDEM_PRODUTO, parameters, resultadoPesquisaDRAutorDemandaReciboOrdemProdutoMapper);
    }

    public List<ResultadoRelatorioDataRetornoDemandaDTO> resultadoAlterarDataRetornoCsv(RelatorioDataRetornoAutorDemandaFilterDTO filterDTO) {

        DynamicParameters parameters = DynamicParameters.get();
        Optional.ofNullable(filterDTO).ifPresent( dto -> {
            parametersAppendFilter(parameters, dto.getDataRetornoAtual(), "dataRetornoAtual");
            parametersAppendFilter(parameters, dto.getCodigoAlteracaoDataRetorno(), "codigoAlteracaoDataRetorno");
        });

        return autorDemandaRepository.findAllNative(SOURCE_RELATORIO_DR_AUTOR_DEMANDA, parameters, relatorioDataRetornoDemandaMapper);
    }


    public List<ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTO> pesquisaDRAutorDemandaADRPorCodigo(Long codigoAlteracaoDataRetorno) {
        DynamicParameters parameters = DynamicParameters.get();
        parametersAppendFilter(parameters, codigoAlteracaoDataRetorno, "codigoAlteracaoDataRetorno");
        return autorDemandaRepository.findAllNative(SOURCE_PESQUISA_DR_AD_ALTERACAO_DT_RETORNO, parameters, resultadoPesquisaDRAutorDemandaReciboOrdemProdutoMapper);
    }


    @Transactional(propagation = Propagation.REQUIRED)
    public SalvarAlteracaoDataRetornoDTO salvarAlteracaoDataRetorno(SalvarAlteracaoDataRetornoDTO dto) {
        serviceValidation.validate(dto.getAlteracaoDataRetorno().getNovaDataRetorno(), dto.getAlteracaoDataRetorno().getDataRetornoAnterior());
        AlteracaoDataRetornoDTO alteracaoDRSalvo = incluir(dto.getAlteracaoDataRetorno());

        salvarAlteracaoDataRetornoProduto(alteracaoDRSalvo, pesquisaDRAutorDemandaReciboOrdemProduto(dto.getPesquisarDataRetornoAutorReciboOrdemProdutoFilter()));
        return new SalvarAlteracaoDataRetornoDTO(alteracaoDRSalvo);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluirAlteracaoDataRetorno(Long id) {
        alteracaoDataRetornoProdutoService.excluirPorAlteracaoDataRetornoId(id);
        excluirPorIdEStatusId(id);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public AlteracaoDataRetornoDTO alterarDataRetorno(Long id, SalvarAlteracaoDataRetornoDTO dto) {
        AlteracaoDataRetornoDTO alteracaoDataRetorno = Optional.of(dto.getAlteracaoDataRetorno()).orElseThrow(RuntimeException::new);
        List<Date> listDataRetornoAnteriores = pesquisaDRAutorDemandaADRPorCodigo(id).stream()
                                                    .map(a -> a.getDataRetornoAnterior())
                                                    .collect(Collectors.toList());
        listDataRetornoAnteriores.forEach(dataRetornoAnterior -> {
            validarNovaDataRetorno(alteracaoDataRetorno, dataRetornoAnterior);
        });
        alterarDataRetornoReciboOrdemProduto(id, alteracaoDataRetorno);
        return alterar(id, alteracaoDataRetorno);
    }

    public void validarNovaDataRetorno(ValidarAlteracaoNovaDataRetornoFilterDTO dto) {
        serviceValidation.validate(dto.getNovaDataRetorno(), dto.getDataRetornoAnterior());
    }

    public ArquivoBase64DTO gerarRelatorioCsv(RelatorioDataRetornoAutorDemandaFilterDTO filterDTO) {
        List<ResultadoRelatorioDataRetornoDemandaDTO> resultados = this.resultadoAlterarDataRetornoCsv(filterDTO);
        return alteracaoDataRetornoReportBuilder.gerarRelatorioCsv(resultados);
    }

    public ArquivoBase64DTO gerarRelatorioPdf(Long codigoAlteracaoDataRetorno) {
        return alteracaoDataRetornoReportBuilder.gerarRelatorioPdf(
            REPORT_RELATORIO_DATA_RETORNO,
            obterEntityPorId(codigoAlteracaoDataRetorno),
            pesquisaDRAutorDemandaADRPorCodigo(codigoAlteracaoDataRetorno));
    }

    public void validarNovaDataRetornoEditar(Long id, ValidarAlteracaoNovaDataRetornoFilterDTO dto) {
        Date novaDataRetorno = Optional.of(dto.getNovaDataRetorno()).orElseThrow(RuntimeException::new);
        List<Date> listDataRetornoAnteriores = pesquisaDRAutorDemandaADRPorCodigo(id).stream()
                                                    .map(a -> a.getDataRetornoAnterior())
                                                    .collect(Collectors.toList());
        listDataRetornoAnteriores.forEach(dataRetornoAnterior -> {
            validarNovaDataRetornoEditar(novaDataRetorno, dataRetornoAnterior);
        });
    }

    private AlteracaoDataRetorno obterEntityPorId(Long id) {
        Optional<AlteracaoDataRetorno> one = repository.findById(id);
        return one.orElseThrow(ResourceNotFoundException::new);
    }

    private void salvarAlteracaoDataRetornoProduto(AlteracaoDataRetornoDTO alteracaoDRSalvo, List<ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTO> resultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTOS) {
        resultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTOS.forEach(dtoResultado -> {
            AlteracaoDataRetornoProdutoDTO alteracaoDataRetornoProdutoDTO = parseToAlteracaoDataRetornoProdutoDTO(dtoResultado, alteracaoDRSalvo);
            alteracaoDataRetornoProdutoService.incluir(alteracaoDataRetornoProdutoDTO);
        });
    }

    private AlteracaoDataRetornoProdutoDTO parseToAlteracaoDataRetornoProdutoDTO(ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoDTO dtoResultado, AlteracaoDataRetornoDTO alteracaoDRSalvo) {
        return AlteracaoDataRetornoProdutoDTO.builder()
            .dataRetornoAnterior(dtoResultado.getDataRetornoAnterior())
            .codigoReciboOrdemProduto(dtoResultado.getCodigoReciboOrdemProduto())
            .alteracaoDataRetorno(alteracaoDRSalvo).build();
    }

    private AlteracaoDataRetornoDTO incluir(AlteracaoDataRetornoDTO dto) {
        dto.setDataCadastro(new Date());
        dto.setStatus(new StatusAlteracaoDataRetornoDTO(1L));
        AlteracaoDataRetorno entity = mapper.toEntity(dto);
        return mapper.toDto(repository.saveAndFlush(entity));
    }

    private void excluirPorIdEStatusId(Long id) {
        AlteracaoDataRetorno entity = obterEntityPorId(id);
        repository.deleteByIdAndStatusId(entity.getId(), 1L);
    }


    private void alterarDataRetornoReciboOrdemProduto(Long id, AlteracaoDataRetornoDTO dto) {
        Optional.ofNullable(obterResultadoAlterarDataRetorno(id))
            .ifPresent(resultado -> {
                resultado.forEach(r -> this.alterarDataRetornoReciboOrdemProduto(r, dto));
            });
    }

    private void alterarDataRetornoReciboOrdemProduto(ResultadoAlterarDataRetornoDTO dtoResultadoAlterarDataRetorno, AlteracaoDataRetornoDTO dto) {
        reciboOrdemProdutoService.alterar(dtoResultadoAlterarDataRetorno.getCodigoReciboOrdemProduto(), new ReciboOrdemProdutoDTO(dto.getNovaDataRetorno()));
    }

    private void validarNovaDataRetorno(AlteracaoDataRetornoDTO dto, Date dataRetornoAnterior) {
        serviceValidation.validate(dto.getNovaDataRetorno(), dataRetornoAnterior);
    }

    private void validarNovaDataRetornoEditar(Date novaDataRetorno, Date dataRetornoAnterior) {
        serviceValidation.validate(novaDataRetorno, dataRetornoAnterior);
    }

    private AlteracaoDataRetornoDTO alterar(Long id, AlteracaoDataRetornoDTO dto) {
        AlteracaoDataRetorno entity = obterEntityPorId(id);
        dto.setId(id);
        dto.setDataCadastro(entity.getDataCadastro());
        dto.setStatus(new StatusAlteracaoDataRetornoDTO(3L));
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    private DynamicParameters getParametersPesquisaDataRetornoAutorReciboOrdemProdutoFilter(PesquisaDRAutorDemandaReciboOrdemProdutoFilterDTO dtoFilter) {
        DynamicParameters parameters = DynamicParameters.get();
        Optional.ofNullable(dtoFilter).ifPresent( dto -> {
            parametersAppendFilter(parameters, dto.getDataRetorno(), "dataRetornoAnterior");
            parametersAppendFilter(parameters, dto.getCodigosAutoresDemanda(), "codigosAutoresDemanda");
        });
        return parameters;
    }

    private DynamicParameters getParametersFilterDTO(PageableRequestModel<PesquisaDRAutorDemandaReciboFilterDTO> pageableModel) {
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            parametersAppendFilter(parameters, filter.getDataRetorno(), "dataRetorno");
            parametersAppendFilter(parameters, filter.getProtocolo(), "protocolo");
            parametersAppendFilter(parameters, filter.getProcesso(), "processo");
            parametersAppendLikeFilter(parameters, filter.getAutor(), "autor");
            parametersAppendFilter(parameters, filter.getIdsUnidadesDispensadoras(), "idsUnidadesDispensadoras");
            parametersAppendFilter(parameters, filter.getCodigoTipoDemanda(), "codigoTipoDemanda");
        });
        return parameters;
    }

    private void parametersAppendFilter(DynamicParameters parameters, Object objFilter, String field) {
        Optional.ofNullable(objFilter).ifPresent(
            filter -> parameters.append(field, filter)
        );
    }

    private void parametersAppendLikeFilter(DynamicParameters parameters, Object objFilter, String field) {
        Optional.ofNullable(objFilter).ifPresent(
            filter -> parameters.appendLike(field, filter.toString())
        );
    }
}
