package br.gov.datasus.scodes.service;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ArquivoDTO;
import br.gov.datasus.scodes.model.dto.DiretorioDTO;
import br.gov.datasus.scodes.model.entity.Arquivo;
import br.gov.datasus.scodes.model.entity.AutorDemanda;
import br.gov.datasus.scodes.model.entity.Diretorio;
import br.gov.datasus.scodes.model.mapper.ArquivoMapper;
import br.gov.datasus.scodes.model.mapper.AutorDemandaMapper;
import br.gov.datasus.scodes.repository.ArquivoRepository;
import br.gov.datasus.scodes.repository.AutorDemandaRepository;
import br.gov.datasus.scodes.repository.DiretorioRepository;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ArquivoService {

    private final ArquivoRepository repository;
    private final DiretorioRepository diretorioRepository;
    private final AutorDemandaRepository autorDemandaRepository;

    private final ArquivoMapper mapper;
    private final AutorDemandaMapper autorDemandaMapper;

    public ArquivoDTO obterPorId(Long id) {
        Optional<Arquivo> one = repository.findById(id);
        Arquivo entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ArquivoDTO> obterArquivosPorDiretorioId(Long diretorioId, Long autorDemandaId) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("diretorioId", diretorioId);
        parameters.append("autorDemandaId", autorDemandaId);
        List<Arquivo> itens = repository.findAll(Arquivo.OBTER_ARQUIVOS_POR_DIRETORIO_ID, parameters, Arquivo.class);
        return mapper.toDto(itens);
    }

    public List<DiretorioDTO> obterDiretorios(Long autorDemandaId) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("autorDemandaId", autorDemandaId);
        Iterable<Diretorio> itens = diretorioRepository.findAll();
        List<DiretorioDTO> diretorios = new ArrayList<DiretorioDTO>();
        for (Diretorio diretorio : itens) {
            DiretorioDTO dto = new DiretorioDTO();
            dto.setId(diretorio.getId());
            dto.setDescricao(diretorio.getDescricao());
            dto.setQuantidadeArquivos(diretorio.getQuantidadeArquivos());
            diretorios.add(dto);
        }
        return diretorios;
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ArquivoDTO incluir(final ArquivoDTO dto) {
        return mapper.toDto(validateAndSave(dto));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ArquivoDTO alterar(final ArquivoDTO dto) {
        return mapper.toDto(validateAndSave(dto));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        Arquivo entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        repository.delete(entity);
    }

    private Arquivo validateAndSave(final ArquivoDTO dto) {
        final Arquivo arquivo = mapper.toEntity(dto);

        Optional.ofNullable(arquivo.getAutorDemanda()).map(AutorDemanda::getId).orElseThrow(() -> new BusinessException(MessageProperty.DEMANDA_OBRIGATORIO));
        Optional.ofNullable(arquivo.getDiretorio()).map(Diretorio::getId).orElseThrow(() -> new BusinessException(MessageProperty.DIRETORIO_OBRIGATORIO));
        Optional.ofNullable(arquivo.getDataProtocolo()).orElseThrow(() -> new BusinessException(MessageProperty.DATA_PROTOCOLO_OBRIGATORIO));
        Optional.ofNullable(arquivo.getArquivo()).orElseThrow(() -> new BusinessException(MessageProperty.ARQUIVO_OBRIGATORIO));

        if (arquivo.getDataInclusao() == null) {
            arquivo.setDataInclusao(Calendar.getInstance().getTime());
        }

        autorDemandaRepository.findById(dto.getAutorDemanda().getId()).orElseThrow(ResourceNotFoundException::new);
        diretorioRepository.findById(dto.getDiretorio().getId()).orElseThrow(ResourceNotFoundException::new);

        return repository.save(arquivo);
    }

}
