package br.gov.datasus.scodes.service;


import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.AutorDTO;
import br.gov.datasus.scodes.model.dto.AutorDemandaDTO;
import br.gov.datasus.scodes.model.dto.DemandaDTO;
import br.gov.datasus.scodes.model.dto.ProcessualDTO;
import br.gov.datasus.scodes.model.dto.filter.DemandaFilterDTO;
import br.gov.datasus.scodes.model.entity.AutorDemanda;
import br.gov.datasus.scodes.model.mapper.AutorDemandaMapper;
import br.gov.datasus.scodes.repository.AutorDemandaRepository;
import br.gov.datasus.scodes.util.AESEncryptionDecryptionUtil;
import br.gov.datasus.scodes.util.ParameterUtil;
import br.gov.datasus.scodes.validation.DemandaServiceValidation;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.Set;

import static java.util.Objects.nonNull;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class AutorDemandaService {

    private final AutorDemandaRepository repository;
    private final AutorDemandaMapper mapper;

    private final DemandaServiceValidation demandaServiceValidation;
    private final AutorService autorService;
    private final DemandaService demandaService;
    private final ProcessualService processualService;

    private final ParameterUtil parameterUtil;
    private final AESEncryptionDecryptionUtil encryptionDecryptionUtil;

    public AutorDemandaDTO obterPorId(Long id) {
        AutorDemanda entity = obterEntityPorId(id);
        AutorDemandaDTO autorDemandaDTO = mapper.toDto(entity);
        String decryptSenhaProcesso = encryptionDecryptionUtil.decrypt(autorDemandaDTO.getDemanda().getSenhaProcesso(), AESEncryptionDecryptionUtil.SECRET);
        autorDemandaDTO.getDemanda().setSenhaProcesso(decryptSenhaProcesso);
        return autorDemandaDTO;
    }

    public List<AutorDemandaDTO> obterTodos() {
        Iterable<AutorDemanda> itens = repository.findAll();
        return mapper.toDto(itens);
    }


    public PageableResponseModel<AutorDemandaDTO> pesquisarPaginado(PageableRequestModel<DemandaFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "AutorDemanda");

        final DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            parameterUtil.append(parameters,filter.getProtocolo(), "protocolo");
            parameterUtil.append(parameters,filter.getProcesso(), "processo");
            parameterUtil.append(parameters,filter.getNomeAutor(), "nomeAutor");
            parameterUtil.append(parameters,filter.getDocumento(), "documento");
            parameterUtil.append(parameters,filter.getNomeMae(), "nomeMae");
            parameterUtil.append(parameters,filter.getIdUnidadeOrganizacional(), "idUnidadeOrganizacional");
            parameterUtil.append(parameters,filter.getIdTipoDemanda(), "idTipoDemanda");
        });

        PageableResponseModel<AutorDemanda> pageableResponseModel = repository.findAll(AutorDemanda.PESQUISAR_PAGINADO, parameters, pagination, AutorDemanda.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }


    @Transactional(propagation= Propagation.REQUIRED)
    public AutorDemandaDTO validateAndSave(final AutorDemandaDTO dto) {
        demandaServiceValidation.validate(dto);

        final Set<AutorDTO> autoresDTO = autorService.salvar(dto.getAutores());
        final DemandaDTO demandaDTO = demandaService.salvar(dto.getDemanda());

        dto.setAutores(autoresDTO);
        dto.setDemanda(demandaDTO);

        AutorDemandaDTO autorDemandaDTO = salvar(dto);

        dto.getProcessual().setAutorDemanda(autorDemandaDTO);
        final ProcessualDTO processualDTO = processualService.salvar(dto.getDemanda().getTipoDemanda(), dto.getProcessual());

        return autorDemandaDTO;
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public AutorDemandaDTO salvar(AutorDemandaDTO dto) {
        return nonNull(dto.getId()) ? alterar(dto) : incluir(dto);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        // TODO: verificar ordem de dispesação
        excluirProcessual(id);
        excluirAutorDemanda(id);
    }


    private void excluirAutorDemanda(final Long id) {
        AutorDemanda entity = obterEntityPorId(id);
        repository.delete(entity);
    }


    private void excluirProcessual(final Long id) {
        processualService.excluirPorAutorDemandaId(id);
    }


    private AutorDemandaDTO incluir(AutorDemandaDTO dto) {
        AutorDemanda entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    private AutorDemandaDTO alterar(AutorDemandaDTO dto) {
        AutorDemanda entity = obterEntityPorId(dto.getId());
        dto.setId(dto.getId());
        mapper.fromDto(dto, entity);
        return mapper.toDto(repository.save(entity));
    }

    private AutorDemanda obterEntityPorId(Long id) {
        Optional<AutorDemanda> one = repository.findById(id);
        return one.orElseThrow(ResourceNotFoundException::new);
    }
}
