package br.gov.datasus.scodes.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import br.gov.datasus.scodes.model.dto.TipoDemandaDTO;
import br.gov.datasus.scodes.model.entity.Autor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.ProcessualDTO;
import br.gov.datasus.scodes.model.entity.AutorDemanda;
import br.gov.datasus.scodes.model.entity.Demanda;
import br.gov.datasus.scodes.model.entity.Processual;
import br.gov.datasus.scodes.model.enums.TipoAcaoContraEnum;
import br.gov.datasus.scodes.model.enums.TipoDemandaEnum;
import br.gov.datasus.scodes.model.mapper.ProcessualMapper;
import br.gov.datasus.scodes.repository.AutorDemandaRepository;
import br.gov.datasus.scodes.repository.CidadeRepository;
import br.gov.datasus.scodes.repository.ProcessualRepository;
import lombok.RequiredArgsConstructor;

import static java.util.Objects.nonNull;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ProcessualService {

    private final ProcessualRepository repository;
    private final CidadeRepository cidadeRepository;
    private final AutorDemandaRepository autorDemandaRepository;
    private final ProcessualMapper mapper;

    public ProcessualDTO obterPorId(Long id) {
        Optional<Processual> one = repository.findById(id);
        Processual entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public ProcessualDTO obterPorAutorDemandaId(Long id){
        Optional<Processual> one = repository.findByAutorDemandaId(id);
        Processual entity = one.orElse(null);
        return mapper.toDto(entity);
    }

    public List<ProcessualDTO> obterTodos() {
        Iterable<Processual> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ProcessualDTO salvar(final ProcessualDTO dto) {
        return nonNull(dto.getId()) ? alterar(dto) : incluir(dto);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public ProcessualDTO salvar(final TipoDemandaDTO tipoDemandaDTO, final ProcessualDTO dto) {
        return tipoDemandaDTO.getId().equals(TipoDemandaEnum.JEFAZ.getCodigo().longValue()) ? salvar(dto) : null;
    }


    private ProcessualDTO incluir(final ProcessualDTO dto) {
        Processual entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    private ProcessualDTO alterar(final ProcessualDTO dto) {
        Processual entity = obterEntityPorId(dto.getId());
        dto.setId(dto.getId());
        mapper.fromDto(dto, entity);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        Processual entity = obterEntityPorId(id);
        repository.delete(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluirPorAutorDemandaId(final Long id) {
        Optional<Processual> one = repository.findByAutorDemandaId(id);
        Processual entity = one.orElse(null);
        if (nonNull(entity)) repository.deleteByAutorDemanda(entity.getAutorDemanda());
    }



    private Processual validateAndSave(final ProcessualDTO dto) {

        final Processual processual = mapper.toEntity(dto);

        Optional.ofNullable(processual).orElseThrow(() -> new BusinessException(MessageProperty.PROCESSUAL_OBRIGATORIO));
        Optional.ofNullable(processual.getSegredo()).orElseThrow(() -> new BusinessException(MessageProperty.DATA_ENTRADA_OBRIGATORIO));
        Optional.ofNullable(processual.getTipoAcaoContra()).orElseThrow(() -> new BusinessException(MessageProperty.ACAO_CONTRA_OBRIGATORIO));
        Optional.ofNullable(processual.getPrazo()).orElseThrow(() -> new BusinessException(MessageProperty.PRAZO_OBRIGATORIO));
        Optional.ofNullable(processual.getVara()).orElseThrow(() -> new BusinessException(MessageProperty.VARA_OBRIGATORIO));
        Optional.ofNullable(processual.getAutorDemanda()).orElseThrow(() -> new BusinessException(MessageProperty.DEMANDA_OBRIGATORIO));
        Optional.ofNullable(processual.getAutorDemanda().getDemanda()).orElseThrow(() -> new BusinessException(MessageProperty.DEMANDA_OBRIGATORIO));

        if (processual.getTipoAcaoContra().getId() == null) {
            throw new BusinessException(MessageProperty.ACAO_CONTRA_OBRIGATORIO);
        }

        if (processual.getPrazo().getId() == null) {
            throw new BusinessException(MessageProperty.PRAZO_OBRIGATORIO);
        }

        if (processual.getAutorDemanda().getDemanda().getId() == null) {
            throw new BusinessException(MessageProperty.DEMANDA_OBRIGATORIO);
        }

        AutorDemanda autorDemanda = autorDemandaRepository.findById(dto.getAutorDemanda().getId()).orElseThrow(ResourceNotFoundException::new);
        Demanda demanda = autorDemanda.getDemanda();

        if (demanda.getTipoDemanda().getId().equals(TipoDemandaEnum.JUDICIAL.getCodigo().longValue())) {
            Optional.ofNullable(demanda.getProcesso()).orElseThrow(() -> new BusinessException(MessageProperty.PROCESSO_OBRIGATORIO));
        } else if (demanda.getTipoDemanda().getId().equals(TipoDemandaEnum.JEFAZ.getCodigo().longValue())) {
            validarTipoAcaoContra(processual);
        }

        return repository.save(processual);
    }

    private void validarTipoAcaoContra(final Processual processual) {
        Optional.ofNullable(processual.getTipoAcaoContra()).orElseThrow(() -> new BusinessException(MessageProperty.ACAO_CONTRA_OBRIGATORIO));
        if (processual.getTipoAcaoContra().equals(TipoAcaoContraEnum.ESTADO_MUNICIPIO)
                || processual.getTipoAcaoContra().equals(TipoAcaoContraEnum.ESTADO_MUNICIPIO_UNIAO)
                || processual.getTipoAcaoContra().equals(TipoAcaoContraEnum.MUNICIPIO)
                || processual.getTipoAcaoContra().equals(TipoAcaoContraEnum.MUNICIPIO_UNIAO)) {

            Optional.ofNullable(processual.getCidade()).orElseThrow(() -> new BusinessException(MessageProperty.CIDADE_OBRIGATORIO));
            processual.setCidade(cidadeRepository.findById(processual.getCidade().getId())
                    .orElseThrow(() -> new BusinessException(MessageProperty.CIDADE_OBRIGATORIO)));
        }
    }


    private Processual obterEntityPorId(Long id) {
        Optional<Processual> one = repository.findById(id);
        return one.orElseThrow(ResourceNotFoundException::new);
    }

}
