package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.UsuarioLocalCadastroDTO;
import br.gov.datasus.scodes.model.dto.UsuarioLocalDTO;
import br.gov.datasus.scodes.model.dto.UsuarioLocalSenhaDTO;
import br.gov.datasus.scodes.model.dto.filter.UsuarioLocalFilterDTO;
import br.gov.datasus.scodes.model.entity.UsuarioLocal;
import br.gov.datasus.scodes.model.mapper.UsuarioLocalMapper;
import br.gov.datasus.scodes.repository.UsuarioLocalRepository;
import lombok.RequiredArgsConstructor;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class UsuarioLocalService {

    private final UsuarioLocalRepository repository;
    private final UsuarioLocalMapper mapper;

    public UsuarioLocalDTO obterPorId(Long id) {
        UsuarioLocal entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<UsuarioLocalDTO> obterTodosPaginado(PageableRequestModel<UsuarioLocalFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "ul");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getNomeLogin())
                .ifPresent(nomeLogin -> parameters.appendLike("nomeLogin", nomeLogin));
            Optional.ofNullable(filter.getNomeUsuario())
                .ifPresent(nomeUsuario -> parameters.appendLike("nomeUsuario", nomeUsuario));
        });
        PageableResponseModel<UsuarioLocal> pageableResponseModel = repository.findAll(UsuarioLocal.OBTER_TODOS_PAGINADO, parameters, pagination, UsuarioLocal.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public UsuarioLocalDTO incluir(UsuarioLocalDTO dto) {
        if (repository.countByNomeLoginIgnoreCase(dto.getNomeLogin()) > 0) {
            throw new BusinessException(MessageProperty.LOGIN_JA_UTILIZADO_USUARIO);
        }
        if (repository.countByDescricaoEmailIgnoreCase(dto.getDescricaoEmail()) > 0) {
            throw new BusinessException(MessageProperty.EMAIL_JA_UTILIZADO_USUARIO);
        }
        UsuarioLocal entity = mapper.toEntity(dto);
        if (dto.getSenhaAutomatica() != null && dto.getSenhaAutomatica()) {
            entity.setDescricaoSenha(generateRandomPassword());
        }
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public UsuarioLocalDTO alterar(Long id, UsuarioLocalDTO dto) {
        UsuarioLocal entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        if (repository.countByNomeLoginIgnoreCaseAndIdNot(dto.getNomeLogin(), id) > 0) {
            throw new BusinessException(MessageProperty.LOGIN_JA_UTILIZADO_USUARIO);
        }
        if (repository.countByDescricaoEmailIgnoreCaseAndIdNot(dto.getDescricaoEmail(), id) > 0) {
            throw new BusinessException(MessageProperty.EMAIL_JA_UTILIZADO_USUARIO);
        }
        mapper.fromDto(dto, entity);
        if (dto.getSenhaAutomatica() != null && dto.getSenhaAutomatica()) {
            entity.setDescricaoSenha(generateRandomPassword());
        }
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public UsuarioLocalDTO alterar(Long id, UsuarioLocalCadastroDTO dto) {
        UsuarioLocal entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (repository.countByDescricaoEmailIgnoreCaseAndIdNot(dto.getEmail(), id) > 0) {
            throw new BusinessException(MessageProperty.EMAIL_JA_UTILIZADO_USUARIO);
        }
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public UsuarioLocalDTO alterar(Long id, UsuarioLocalSenhaDTO dto) {
        UsuarioLocal entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        if (!hashPassword(dto.getSenhaAtual()).equals(entity.getDescricaoSenha())) {
            throw new BusinessException(MessageProperty.SENHA_ATUAL_INVALIDA);
        }
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(Long id) {
        UsuarioLocal entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        repository.delete(entity);
    }

    public static String hashPassword(String password) {
        return DigestUtils.sha256Hex(password);
    }

    public static String generateRandomPassword() {
        return RandomStringUtils.randomAlphanumeric(8);
    }
}
