package br.gov.datasus.scodes.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import br.gov.datasus.scodes.model.dto.TipoVaraDTO;
import br.gov.datasus.scodes.model.entity.TipoVara;
import br.gov.datasus.scodes.model.mapper.TipoVaraMapper;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.CidadeDTO;
import br.gov.datasus.scodes.model.dto.VaraDTO;
import br.gov.datasus.scodes.model.dto.filter.VaraFilterDTO;
import br.gov.datasus.scodes.model.entity.Cidade;
import br.gov.datasus.scodes.model.entity.Vara;
import br.gov.datasus.scodes.model.mapper.CidadeMapper;
import br.gov.datasus.scodes.model.mapper.VaraMapper;
import br.gov.datasus.scodes.repository.CidadeRepository;
import br.gov.datasus.scodes.repository.TipoVaraRepository;
import br.gov.datasus.scodes.repository.VaraRepository;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class VaraService {

    private final VaraRepository repository;
    private final CidadeRepository cidadeRepository;
    private final TipoVaraRepository tipoVaraRepository;

    private final VaraMapper mapper;
    private final TipoVaraMapper tipoVaraMapper;
    private final CidadeMapper cidadeMapper;

    public VaraDTO obterPorId(Long id) {
        Optional<Vara> one = repository.findById(id);
        Vara entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<VaraDTO> obterTodos() {
        Iterable<Vara> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public List<CidadeDTO> pesquisarCidadeVara(String uf, String nome) {
        final DynamicParameters parameters = DynamicParameters.get();
        parameters.append("uf", uf);
        parameters.appendLike("nome", nome);
        Iterable<Cidade> itens = cidadeRepository.findAll(Cidade.PESQUISAR_CIDADE_VARA, parameters, Cidade.class);
        return cidadeMapper.toDto(itens);
    }

    public List<VaraDTO> pesquisarTipoVaraEVaraPorNomeCidade(Long cidadeId, String nome) {
        final DynamicParameters parameters = DynamicParameters.get();
        parameters.append("cidadeId", cidadeId);
        parameters.appendLike("nome", nome);
        Iterable<Vara> itens = repository.findAll(Vara.PESQUISAR_TIPO_VARA_E_POR_NOME_CIDADE, parameters, Vara.class);
        return mapper.toDto(itens);
    }

    public List<TipoVaraDTO> obterTiposVaraPorCodigoMunicipio(Long cidadeId) {
        List<TipoVara> tiposVaras  = repository.obterTiposVaraPorCodigoMunicipio(cidadeId);
        return tipoVaraMapper.toDto(tiposVaras);
    }

    public List<VaraDTO> obterVarasPorCodigoTipoVara(Long tipoVaraId) {
        return mapper.toDto(repository.findByTipoVaraId(tipoVaraId));
    }

    public PageableResponseModel<VaraDTO> pesquisarPaginado(PageableRequestModel<VaraFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "Vara");
        final DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getDescricao())
            .ifPresent(descricao -> parameters.append("descricao", descricao));
            Optional.ofNullable(filter.getCidade().getId())
            .ifPresent(idCidade -> parameters.append("cidadeId", idCidade));
        });

        final PageableResponseModel<Vara> pageableResponseModel = repository.findAll(Vara.OBTER_TODOS_PAGINADO, parameters, pagination, Vara.class);

        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public VaraDTO incluir(final VaraDTO dto) {
        final Vara entity = mapper.toEntity(dto);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public VaraDTO alterar(final Long id, final VaraDTO dto) {
        final Optional<Vara> one = repository.findById(id);
        final Vara entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        return mapper.toDto(validateAndSave(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        final Optional<Vara> one = repository.findById(id);
        final Vara entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }

    private Vara validateAndSave(final Vara vara) {

        Optional.ofNullable(vara.getDescricao()).orElseThrow(() -> new BusinessException(MessageProperty.DESCRICAO_OBRIGATORIO));
        Optional.ofNullable(vara.getEndereco()).orElseThrow(() -> new BusinessException(MessageProperty.ENDERECO_OBRIGATORIO));
        Optional.ofNullable(vara.getCidade()).orElseThrow(() -> new BusinessException(MessageProperty.CIDADE_OBRIGATORIO));
        Optional.ofNullable(vara.getCidade().getId()).orElseThrow(() -> new BusinessException(MessageProperty.CIDADE_OBRIGATORIO));
        Optional.ofNullable(vara.getBairro()).orElseThrow(() -> new BusinessException(MessageProperty.BAIRRO_OBRIGATORIO));
        Optional.ofNullable(vara.getForum()).orElseThrow(() -> new BusinessException(MessageProperty.FORUM_OBRIGATORIO));
        Optional.ofNullable(vara.getTipoVara()).orElseThrow(() -> new BusinessException(MessageProperty.TIPO_VARA_OBRIGATORIO));

        vara.setTipoVara(
                tipoVaraRepository.findById(vara.getTipoVara().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.TIPO_VARA_OBRIGATORIO))
                );
        vara.setCidade(
                cidadeRepository.findById(vara.getCidade().getId())
                .orElseThrow(() -> new BusinessException(MessageProperty.CIDADE_OBRIGATORIO))
                );

        return repository.save(vara);
    }

    public List<CidadeDTO> obterMunicipiosDaVaraPorNome(String nome, String uf) {
        return this.pesquisarCidadeVara(uf, nome);
    }
}
