package br.gov.datasus.scodes.validation;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.AutorDemandaDTO;
import br.gov.datasus.scodes.model.dto.DemandaDTO;
import br.gov.datasus.scodes.model.dto.ProtocoloJefazDTO;
import br.gov.datasus.scodes.model.dto.filter.DemandaFilterDTO;
import br.gov.datasus.scodes.model.enums.TipoDemandaEnum;
import br.gov.datasus.scodes.service.DemandaService;
import br.gov.datasus.scodes.service.ProtocoloJefazService;
import br.gov.datasus.scodes.util.AESEncryptionDecryptionUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.Calendar;

import static br.gov.datasus.scodes.configuration.properties.MessageProperty.*;
import static java.util.Objects.isNull;
import static java.util.Objects.nonNull;
import static java.util.Optional.ofNullable;

@Service
@RequiredArgsConstructor
public class DemandaServiceValidation {

    private static final String DESCRICAO_PERFIL_LOGIN = "Administrador/Scodes";
    private final DemandaService demandaService;
    private final ProtocoloJefazService protocoloJefazService;
    private final AESEncryptionDecryptionUtil encryptionDecryptionUtil;

    public void validate(AutorDemandaDTO dto) {

        DemandaDTO demandaDTO = dto.getDemanda();
        demandaValidate(demandaDTO);
        demandaDataEntrandaValidate(demandaDTO);
        demandaTipoDemandaValidate(demandaDTO);
        demandaTipoDemandaIdValidate(demandaDTO);
        demandaUnidadeOrganizacionalValidate(demandaDTO);
        demandaUnidadeOrganizacionalIdValidate(demandaDTO);
        demandaTipoProcessoValidate(demandaDTO);
        demandaJaExisteDemandaComProtocoloProcessoEUnidadeOrganizacionalValidate(demandaDTO);

        autorDemandaAutoresValidate(dto);
        autorDemandaAutoresCidadeValidate(dto);

        setAutorDemandaDataEntradaValidate(dto);
        setDemandaDataEntradaValidate(dto);
        setDemandaDataInclusaoValidate(dto);
        gerarProtocoloJefaz(demandaDTO);
        encryptSenhaProcesso(demandaDTO);
    }

    public void demandaJaExisteDemandaComProtocoloProcessoEUnidadeOrganizacionalValidate(DemandaDTO dto) {
        if (isNull(dto.getId())) {
            DemandaFilterDTO filter = new DemandaFilterDTO();
            filter.setProtocolo(dto.getProtocolo());
            filter.setProcesso(dto.getProcesso());
            filter.setIdUnidadeOrganizacional(dto.getUnidadeOrganizacional().getId());
            demandaService.jaExisteDemandaComProtocoloProcessoEUnidadeOrganizacional(filter);
        }
    }

    private void demandaValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto, DEMANDA_OBRIGATORIO);
    }

    private void demandaDataEntrandaValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto.getDataEntrada(), DATA_ENTRADA_OBRIGATORIO);
    }

    private void demandaTipoDemandaValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto.getTipoDemanda(), TIPO_DEMANDA_OBRIGATORIO);
    }

    private void demandaTipoDemandaIdValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto.getTipoDemanda().getId(), TIPO_DEMANDA_OBRIGATORIO);
    }

    private void demandaUnidadeOrganizacionalValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto.getUnidadeOrganizacional(), UNIDADE_ORGANIZACIONAL_OBRIGATORIO);
    }

    private void demandaUnidadeOrganizacionalIdValidate(DemandaDTO dto) {
        ofNullable(dto.getUnidadeOrganizacional().getId()).orElseThrow(() -> new BusinessException(UNIDADE_ORGANIZACIONAL_OBRIGATORIO));
    }

    private void demandaTipoProcessoValidate(DemandaDTO dto) {
        ofNullableThrowBusinessException(dto.getTipoProcesso(), TIPO_PROCESO_OBRIGATORIO);
    }

    private void autorDemandaAutoresValidate(AutorDemandaDTO dto) {
        if (isNull(dto.getAutores()) || dto.getAutores().isEmpty())
            throw new BusinessException(AUTOR_OBRIGATORIO);
    }

    private void autorDemandaAutoresCidadeValidate(AutorDemandaDTO dto) {
        dto.getAutores().forEach(autor -> {
            if (nonNull(autor) && isNull(autor.getCidade().getId()))
                throw new BusinessException(CIDADE_OBRIGATORIO);
        });
    }

    private void ofNullableThrowBusinessException(Object object, MessageProperty messageProperty) {
        ofNullable(object).orElseThrow(() -> new BusinessException(messageProperty));
    }

    public void setDataValidate(AutorDemandaDTO autorDemandaDTO) {
        setAutorDemandaDataEntradaValidate(autorDemandaDTO);
    }

    private void setAutorDemandaDataEntradaValidate(AutorDemandaDTO autorDemandaDTO) {
        if (isNull(autorDemandaDTO.getDataEntrada())) {
            autorDemandaDTO.setDataEntrada(Calendar.getInstance().getTime());
        }
    }

    private void setDemandaDataEntradaValidate(AutorDemandaDTO autorDemandaDTO) {
        if (isNull(autorDemandaDTO.getDemanda().getDataEntrada())) {
            autorDemandaDTO.getDemanda().setDataEntrada(Calendar.getInstance().getTime());
        }
    }

    private void setDemandaDataInclusaoValidate(AutorDemandaDTO autorDemandaDTO) {
        if (isNull(autorDemandaDTO.getDemanda().getDataInclusao())) {
            autorDemandaDTO.getDemanda().setDataInclusao(Calendar.getInstance().getTime());
        }
    }

    private void gerarProtocoloJefaz(DemandaDTO demandaDTO) {
        if (isTipoDemandaJefaz(demandaDTO) && nonNull(demandaDTO.getSenhaProcesso())) {
            ProtocoloJefazDTO protocoloJefazDTO = getProtocoloJefazDTO(demandaDTO);
            ProtocoloJefazDTO protocoloJefazSalvo = protocoloJefazService.incluir(protocoloJefazDTO);
            String protocoloJefaz = buildProtocoloJefaz(protocoloJefazSalvo);
            demandaDTO.setProtocolo(protocoloJefaz);
        }
    }

    private ProtocoloJefazDTO getProtocoloJefazDTO(DemandaDTO dto) {
        String descricaoPerfilLogin = nonNull(dto.getPerfilModel()) ? dto.getPerfilModel().getDescricao() : DESCRICAO_PERFIL_LOGIN;
        return new ProtocoloJefazDTO(String.valueOf(LocalDate.now().getYear()), descricaoPerfilLogin,
            dto.getUnidadeOrganizacional().getId());
    }

    private String buildProtocoloJefaz(ProtocoloJefazDTO protocoloJefazSalvo) {
        return new StringBuilder()
            .append(protocoloJefazSalvo.getCodigoUnidadeOrganizacional().toString()).append("00")
            .append(protocoloJefazSalvo.getContador().toString()).append("000")
            .append(protocoloJefazSalvo.getAno()).toString();
    }

    private void encryptSenhaProcesso(DemandaDTO demandaDTO) {
        if ((isTipoDemandaJudicial(demandaDTO) && nonNull(demandaDTO.getSenhaProcesso())) || ((isTipoDemandaJefaz(demandaDTO) && nonNull(demandaDTO.getSenhaProcesso())) )) {
            String encryptSenhaProcesso = encryptionDecryptionUtil
                                            .encrypt(demandaDTO.getSenhaProcesso(), AESEncryptionDecryptionUtil.SECRET);
            demandaDTO.setSenhaProcesso(encryptSenhaProcesso);
        }
    }


    private boolean isTipoDemandaJefaz(DemandaDTO demandaDTO) {
        return demandaDTO.getTipoDemanda().getId().equals(TipoDemandaEnum.JEFAZ.getCodigo().longValue());
    }

    private boolean isTipoDemandaJudicial(DemandaDTO demandaDTO) {
        return demandaDTO.getTipoDemanda().getId().equals(TipoDemandaEnum.JUDICIAL.getCodigo().longValue());
    }

}
