import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { MatTableDataSource } from '@angular/material';

import { FuncionalidadeService } from '../../../services/funcionalidade.service';
import { TreeFuncionalidadeService } from '../../../services/tree-funcionalidade.service';
import { FuncionalidadeNodeModel } from '../../../models/funcionalidade-node.model';
import { PerfilFuncionalidadeModel } from '../../../models/perfil-funcionalidade.model';

@Component({
  selector: 'scodes-tree-funcionalidade',
  templateUrl: './tree-funcionalidade.component.html',
  styleUrls: ['./tree-funcionalidade.component.scss'],
  providers: [TreeFuncionalidadeService]
})
export class TreeFuncionalidadeComponent implements OnInit {

  @Input()
  set perfilFuncionalidades(perfilFuncionalidades: PerfilFuncionalidadeModel[]) {
    this.perfilFuncionalidadesInternal = Array.isArray(perfilFuncionalidades) ?
      JSON.parse(JSON.stringify(perfilFuncionalidades)) : [];
  }

  @Output()
  public perfilFuncionalidadeChanged: EventEmitter<PerfilFuncionalidadeModel> = new EventEmitter();

  public displayedColumns: string[];
  public dataSource: MatTableDataSource<FuncionalidadeNodeModel>;

  private treeTable: FuncionalidadeNodeModel[];
  private perfilFuncionalidadesInternal: PerfilFuncionalidadeModel[];

  constructor(
    private funcionalidadeService: FuncionalidadeService,
    private treeFuncionalidadeService: TreeFuncionalidadeService
  ) { }

  public ngOnInit(): void {
    this.treeTable = [];
    this.displayedColumns = [
      'nome',
      'criar',
      'alterar',
      'excluir',
      'pesquisar',
      'imprimir'
    ];

    this.funcionalidadeService.obterTodos()
      .subscribe(data => {
        const tree = this.treeFuncionalidadeService.buildTree(data, this.perfilFuncionalidadesInternal);
        this.treeTable = this.treeFuncionalidadeService.toTreeTable(tree);
        this.dataSource = this.generateDataSource();
      });
  }

  public formatIndentation(node: FuncionalidadeNodeModel, step: number = 5): string {
    return '&nbsp;'.repeat(node.depth * step);
  }

  public onNodeClick(clickedNode: FuncionalidadeNodeModel): void {
    clickedNode.isExpanded = !clickedNode.isExpanded;
    this.treeFuncionalidadeService.walkInTree(clickedNode.children, node => {
        node.isVisible = (clickedNode.isExpanded && node.parent.isExpanded);
    });
    this.dataSource = this.generateDataSource();
  }

  public onCheckboxChange(changedNode: FuncionalidadeNodeModel, property: string): void {
    if (!changedNode.funcionalidade[property]) {
      return;
    }
    changedNode.perfilFuncionalidade[property] = !changedNode.perfilFuncionalidade[property];
    this.perfilFuncionalidadeChanged.emit(changedNode.perfilFuncionalidade);

    this.treeFuncionalidadeService.walkInTree(changedNode.children, node => {
      if (!node.funcionalidade[property]) {
        return;
      }
      node.perfilFuncionalidade[property] = changedNode.perfilFuncionalidade[property];
      this.perfilFuncionalidadeChanged.emit(node.perfilFuncionalidade);
    });

    if (changedNode.parent) {
      this.treeFuncionalidadeService.walkToRoot(changedNode.parent, node => {
        if (!node.funcionalidade[property]) {
          return;
        }
        node.perfilFuncionalidade[property] = !!node.children.filter(v => v.perfilFuncionalidade[property]).length;
        this.perfilFuncionalidadeChanged.emit(node.perfilFuncionalidade);
      });
    }
  }

  public clearChecked() {
    this.treeTable.forEach(node => {
      if (node.perfilFuncionalidade.situacaoPesquisar ||
        node.perfilFuncionalidade.situacaoCriar ||
        node.perfilFuncionalidade.situacaoAlterar ||
        node.perfilFuncionalidade.situacaoExcluir ||
        node.perfilFuncionalidade.situacaoImprimir
      ) {
        Object.assign(node.perfilFuncionalidade, {
          situacaoPesquisar: false,
          situacaoCriar: false,
          situacaoAlterar: false,
          situacaoExcluir: false,
          situacaoImprimir: false,
        });
        this.perfilFuncionalidadeChanged.emit(node.perfilFuncionalidade);
      }
    });
  }

  public isChecked(node: FuncionalidadeNodeModel, property: string): boolean {
    return node.perfilFuncionalidade[property];
  }

  public isIndeterminate(node: FuncionalidadeNodeModel, property: string): boolean {
    return (node.perfilFuncionalidade[property] &&
      node.children.length !== node.children.filter(v => !!v.perfilFuncionalidade[property]).length);
  }

  public isDisabled(node: FuncionalidadeNodeModel, property: string): boolean {
    return !node.funcionalidade[property];
  }

  private generateDataSource(): MatTableDataSource<FuncionalidadeNodeModel> {
    return new MatTableDataSource(this.treeTable.filter(node => node.isVisible));
  }
}
