import { Component, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators, ValidationErrors, ValidatorFn } from '@angular/forms';
import { Observable, Subject } from 'rxjs';
import { map, finalize, takeUntil } from 'rxjs/operators';
import { trigger, transition, style, animate } from '@angular/animations';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel } from '@datasus/core';

import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';
import { AreaEnum } from 'src/app/shared/enums/area.enum';
import { UnidadeDispensadoraModel } from 'src/app/shared/models/unidade-dispensadora.model';

import { UnidadeDispensadoraService } from 'src/app/shared/services/unidade-dispensadora.service';
import { UnidadeOrganizacionalService } from 'src/app/shared/services/unidade-organizacional.service';
import { AreaService } from 'src/app/shared/services/area.service';
import { TipoProfissaoService } from 'src/app/shared/services/tipo-profissao.service';

import { UsuarioLocalModel } from '../../models/usuario-local.model';
import { PerfilService } from '../../../perfil/services/perfil.service';
import { UsuarioLocalService } from '../../services/usuario-local.service';

@Component({
  selector: 'scodes-usuario-form',
  templateUrl: './usuario.form.component.html',
  styleUrls: ['./usuario.form.component.scss'],
  animations: [
    trigger('fadeInOut', [
      transition(':enter', [
        style({ opacity: 0 }),
        animate(500, style({ opacity: 1 }))
      ]),
      transition(':leave', [
        animate(500, style({ opacity: 0 }))
      ])
    ])
  ]
})
export class UsuarioFormComponent implements OnInit, OnDestroy {

  formGroup: FormGroup;
  saving: boolean;

  usuario: UsuarioLocalModel;

  listaTipoConselho: DominioItemModel[];
  listaAreas: DominioItemModel[];
  listaUnidadeOrganizacional: DominioItemModel[];
  listaUnidadeDispensadora: DominioItemModel[];
  listaPerfilJudicial: DominioItemModel[];
  listaPerfilAdm: DominioItemModel[];

  loadingArea = false;
  loadingTipoConselho = false;
  loadingPerfilJudicial = false;
  loadingPerfilAdm = false;
  loadingUnidadeDispensadora = false;
  loadingUnidadeOrganizacional = false;

  private onDestroy$ = new Subject<void>();

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
    private unidadeOrganizacionalService: UnidadeOrganizacionalService,
    private areaService: AreaService,
    private tipoProfissaoService: TipoProfissaoService,
    private perfilService: PerfilService,
    private usuarioLocalService: UsuarioLocalService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Usuário');

    this.buildForm();
    if (this.route.snapshot.data.item) {
      this.editItem(this.route.snapshot.data.item);
    }

    this.obterArea();
    this.obterTipoConselho();
  }

  ngOnDestroy(): void {
    this.onDestroy$.next();
    this.onDestroy$.complete();
  }

  get showUnidadeOrganizacional(): boolean {
    return this.formGroup &&
      Number(this.formGroup.get('area').value) === AreaEnum.UNIDADE_ORGANIZACIONAL;
  }

  get showUnidadeDispensadora(): boolean {
    return this.formGroup &&
      Number(this.formGroup.get('area').value) === AreaEnum.UNIDADE_DISPENSADORA;
  }

  get showSenha(): boolean {
    return (!this.usuario || !this.usuario.id || !!this.formGroup.get('redefinirSenha').value);
  }

  get isRequiredSenha(): boolean {
    return (this.showSenha && !this.formGroup.get('senhaAutomatica').value);
  }

  save(event: Event): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    this.saving = true;

    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO);
        this.router.navigate(['/controle-acesso/usuario/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  resetForm(): void {
    setTimeout(() => {
      this.formGroup.get('situacaoAtivo').setValue(false);
      this.formGroup.get('redefinirSenha').setValue(false);
      this.formGroup.get('senhaAutomatica').setValue(false);
    });
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('email') ? MensagemEnum.EMAIL :
      fc.hasError('minlength') ? MensagemEnum._(MensagemEnum.MIN_LENGTH, fc.getError('minlength')) :
      fc.hasError('confirmPassword') ? MensagemEnum.CONFIRM_PASSWORD :
      '';
  }

  compareDominio(v1: DominioItemModel, v2: DominioItemModel) {
    return (v1 && v2 && v1.codigo === v2.codigo);
  }

  private editItem(item: UsuarioLocalModel): void {
    this.layoutService.setTitle('Alterar Usuário');
    this.usuario = item;
    this.formGroup.get('nomeLogin').setValue(this.usuario.nomeLogin);
    this.formGroup.get('descricaoEmail').setValue(this.usuario.descricaoEmail);
    this.formGroup.get('situacaoAtivo').setValue(this.usuario.situacaoAtivo);
    this.formGroup.get('nomeUsuario').setValue(this.usuario.nomeUsuario);
    this.formGroup.get('numeroTelefone').setValue(this.usuario.numeroTelefone);
    this.formGroup.get('numeroConselho').setValue(this.usuario.numeroConselho);
    if (this.usuario.tipoProfissao) {
      this.formGroup.get('tipoConselho').setValue(String(this.usuario.tipoProfissao.id));
    }

    if (this.usuario.perfis && this.usuario.perfis.length > 0) {
      this.formGroup.get('area').setValue(String(this.usuario.perfis[0].area.id));
      this.formGroup.get('perfilJudicial').setValue(String(this.usuario.perfis[0].id));
    }
    if (this.usuario.perfil) {
      if (this.formGroup.get('area').value == null) {
        this.formGroup.get('area').setValue(String(this.usuario.perfil.area.id));
      }
      this.formGroup.get('perfilAdm').setValue(String(this.usuario.perfil.id));
    }
    if (this.usuario.unidadesDispensadoras && this.usuario.unidadesDispensadoras.length > 0) {
      const unidadesDispensadoras = this.usuario.unidadesDispensadoras
        .map((ud: UnidadeDispensadoraModel) => ({codigo: String(ud.id), descricao: ud.descricao} as DominioItemModel));
      this.formGroup.get('unidadeDispensadora').setValue(unidadesDispensadoras);
    }
    if (this.usuario.unidadeOrganizacional) {
      this.formGroup.get('unidadeOrganizacional').setValue(String(this.usuario.unidadeOrganizacional.id));
    }
  }

  private request(): Observable<UsuarioLocalModel> {
    const value = this.formGroup.value;

    const usuarioLocal = {
      nomeLogin: value.nomeLogin,
      nomeUsuario: value.nomeUsuario,
      descricaoEmail: value.descricaoEmail,
      numeroTelefone: value.numeroTelefone,
      numeroConselho: value.numeroConselho,
      situacaoAtivo: value.situacaoAtivo,
      tipoProfissao: value.tipoConselho != null ? { id: Number(value.tipoConselho) } : null,
      unidadeOrganizacional: value.unidadeOrganizacional != null ? { id: Number(value.unidadeOrganizacional) } : null,
      unidadesDispensadoras: value.unidadeDispensadora != null ? value.unidadeDispensadora
        .filter((v: any) => !!v)
        .map((v: any) => ({id: Number(v.codigo)})) : null,
      perfis: value.perfilJudicial != null ? [{id: Number(value.perfilJudicial)}] : null,
      perfil: value.perfilAdm != null ? { id: Number(value.perfilAdm) } : null,
      descricaoSenha: value.descricaoSenha,
      senhaAutomatica: value.senhaAutomatica
    } as UsuarioLocalModel;

    if (this.usuario && this.usuario.id) {
      return this.usuarioLocalService.alterar(usuarioLocal, this.usuario.id);
    }
    return this.usuarioLocalService.incluir(usuarioLocal);
  }

  private buildForm(): void {
    this.formGroup = this.formBuilder.group({
      nomeLogin: [null],
      descricaoEmail: [null, [Validators.email]],
      descricaoSenha: [null, [Validators.minLength(5)]],
      confirmaSenha: [null],
      situacaoAtivo: [false],
      nomeUsuario: [null],
      numeroTelefone: [null],
      tipoConselho: [null],
      numeroConselho: [null],
      area: [null],
      unidadeOrganizacional: [null],
      unidadeDispensadora: [null],
      perfilJudicial: [null],
      perfilAdm: [null],
      redefinirSenha: [false],
      senhaAutomatica: [null],
    }, {
      validator: Validators.compose([
        this.confirmPasswordValidator('descricaoSenha', 'confirmaSenha'),
      ])
    });

    this.formGroup.get('area').valueChanges
      .pipe(takeUntil(this.onDestroy$))
      .subscribe(value => this.changeArea(value));

    this.formGroup.get('senhaAutomatica').valueChanges
      .pipe(takeUntil(this.onDestroy$))
      .subscribe(value => this.changeSenhaAutomatica(value));
  }

  private changeArea(value: any): void {
    this.listaUnidadeOrganizacional = [];
    this.listaUnidadeDispensadora = [];
    this.listaPerfilJudicial = [];
    this.listaPerfilAdm = [];

    this.formGroup.get('unidadeOrganizacional').setValue(null);
    this.formGroup.get('unidadeDispensadora').setValue(null);
    this.formGroup.get('perfilJudicial').setValue(null);
    this.formGroup.get('perfilAdm').setValue(null);

    if (value == null) {
      return;
    }

    switch (Number(value)) {
      case AreaEnum.UNIDADE_ORGANIZACIONAL:
        this.obterUnidadeOrganizacional();
        break;
      case AreaEnum.UNIDADE_DISPENSADORA:
        this.obterUnidadeDispensadora();
        break;
    }

    this.obterPerfilJudicial(value);
    this.obterPerfilAdm(value);
  }

  private changeSenhaAutomatica(value: any) {
    this.formGroup.get('descricaoSenha').setValue(null);
    this.formGroup.get('confirmaSenha').setValue(null);
  }

  private obterArea(): void {
    this.loadingArea = true;
    this.areaService
      .obterDominio()
      .pipe(finalize(() => this.loadingArea = false))
      .subscribe(value => this.listaAreas = value);
  }

  private obterTipoConselho(): void {
    this.loadingTipoConselho = true;
    this.tipoProfissaoService
      .obterDominio()
      .pipe(finalize(() => this.loadingTipoConselho = false))
      .subscribe(value => this.listaTipoConselho = value);
  }

  private obterUnidadeOrganizacional(): void {
    this.loadingUnidadeOrganizacional = true;
    this.unidadeOrganizacionalService
      .obterDominio()
      .pipe(finalize(() => this.loadingUnidadeOrganizacional = false))
      .subscribe(value => this.listaUnidadeOrganizacional = value);
  }

  private obterUnidadeDispensadora(): void {
    this.loadingUnidadeDispensadora = true;
    this.unidadeDispensadoraService
      .obterDominio()
      .pipe(finalize(() => this.loadingUnidadeDispensadora = false))
      .subscribe(value => this.listaUnidadeDispensadora = value);
  }

  private obterPerfilJudicial(idArea: number): void {
    this.loadingPerfilJudicial = true;
    this.perfilService
      .obterDominioPerfilJudicial(idArea)
      .pipe(finalize(() => this.loadingPerfilJudicial = false))
      .subscribe(value => this.listaPerfilJudicial = value);
  }

  private obterPerfilAdm(idArea: number): void {
    this.loadingPerfilAdm = true;
    this.perfilService
      .obterDominioPerfilAdm(idArea)
      .pipe(finalize(() => this.loadingPerfilAdm = false))
      .subscribe(value => this.listaPerfilAdm = value);
  }

  private confirmPasswordValidator(controlName: string, matchingControlName: string): ValidatorFn | null {
    return (formGroup: FormGroup): ValidationErrors | null => {
      const control = formGroup.get(controlName);
      const matchingControl = formGroup.get(matchingControlName);

      if (matchingControl.errors && !matchingControl.errors.confirmPassword) {
        return null;
      }

      if (control.value !== matchingControl.value) {
        matchingControl.setErrors({confirmPassword: true});
      } else {
        matchingControl.setErrors(null);
      }

      return null;
    };
  }

}
