import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { Observable } from 'rxjs';
import { finalize } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';
import { DadosPessoaisService } from '../../services/dados-pessoais.service';
import { MensagemEnum } from '../../../shared/enums/mensagem.enum';
import { UsuarioCadastroModel } from '../../models/usuario-cadastro.model';

@Component({
  selector: 'scodes-atualizacao-cadastro',
  templateUrl: './atualizacao-cadastro.component.html',
})
export class AtualizacaoCadastroComponent implements OnInit {

  formGroup: FormGroup;
  saving = false;

  onMobile$: Observable<boolean>;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private dadosPessoaisService: DadosPessoaisService
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Atualização de Cadastro');

    this.onMobile$ = this.layoutService.onMobile();

    this.formGroup = this.formBuilder.group({
      email: [
        this.route.snapshot.data.userEmail,
        Validators.compose([
          Validators.required,
          Validators.email,
        ])
      ],
    });
  }

  save(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }

    const dados = this.formGroup.value as UsuarioCadastroModel;

    this.saving = true;
    this.dadosPessoaisService.atualizarCadastro(dados).pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => this.dialogService.showNotification(MensagemEnum.REGISTRO_SALVO),
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      fc.hasError('email') ? MensagemEnum.EMAIL :
      '';
  }

}
