import { Component, OnInit, Inject, ViewChild } from '@angular/core';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';

import { FormValidators } from '@datasus/forms';
import { DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { AutorComponent } from '../autor/autor.component';
import { finalize } from 'rxjs/operators';
import { AutorModel } from 'src/app/shared/models/autor.model';
import { Observable } from 'rxjs';
import { AutorService } from '../../services/autor.service';

@Component({
  selector: 'scodes-dialog-autor',
  templateUrl: './dialog-autor.component.html',
  styleUrls: ['./dialog-autor.component.scss']
})
export class DialogAutorComponent implements OnInit {

  @ViewChild(AutorComponent) autorComponent: AutorComponent;

  formGroup: FormGroup;
  isSalvo: boolean;
  isEditar: boolean;
  isAdicionar: boolean;
  dataCadastro: Date;
  autores: AutorModel[];
  autorId: number;

  viewChars = false;
  actionButtonCtrl: Map<AutorModel, boolean> = new Map();

  // Data Table Config
  displayedColumns: string[] = [
    'autor',
    'documento',
    'dataEntrada',
    'status',
    'actions'
  ];

  displayType = {
    titulo: 'Adicionar Autor',
    icone: 'person_add',
  };



  constructor(
    public dialogRef: MatDialogRef<DialogAutorComponent>,
    @Inject(MAT_DIALOG_DATA) public data: any,
    private formBuilder: FormBuilder,
    private dialogService: DialogService,
    private autorService: AutorService,
  ) { }

  ngOnInit() {

    this.autores = this.data.autores;
    this.dataCadastro = new Date();

    this.formGroup = this.formBuilder.group({
      nome: [null, Validators.compose([Validators.required, Validators.minLength(10)])],
      dataNascimento: [null, Validators.required],
      cpf: [null, Validators.compose([Validators.required, FormValidators.cpf])],
      cns: [null],
      rg: [null],
      cep: [null],
      nomeMae: [null],
      sexo: [null],
      logradouro: [null],
      bairro: [null],
      estado: [null],
      cidade: [null, Validators.required],
      numero: [null],
      complemento: [null],
      email: [null, Validators.compose([Validators.email, Validators.minLength(5)])],
      telefone: [null],
      obsTelefone: [null],
      outroTelefone: [null],
      obsOutroTelefone: [null],
    });

  }

  getAutorId(id: number): void {
    this.autorId = id;
  }

  salvarAutor(event: Event) {

    event.preventDefault();
    this.autorComponent.onSubmit();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos obrigatórios');
    }
    this.isSalvo = true;
    this.request().pipe(
      finalize(() => this.isSalvo = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  editar(autor: AutorModel) {

    this.formGroup.reset();
    this.displayType = {
      titulo: 'Editar',
      icone: 'person_outline',
    };

    this.isEditar = true;

    this.autorId = autor.id;
    this.formGroup.get('nome').setValue(autor.nome);
    this.formGroup.get('dataNascimento').setValue(autor.dataNascimento);
    this.formGroup.get('cpf').setValue(autor.cpf);
    this.formGroup.get('cns').setValue(autor.cns);
    this.formGroup.get('rg').setValue(autor.rg);
    this.formGroup.get('cep').setValue(autor.cep);
    this.formGroup.get('nomeMae').setValue(autor.nomeMae);
    this.formGroup.get('sexo').setValue(autor.sexo);
    this.formGroup.get('logradouro').setValue(autor.logradouro);
    this.formGroup.get('bairro').setValue(autor.bairro);
    this.formGroup.get('estado').setValue(autor.cidade.uf);
    this.formGroup.get('cidade').setValue(autor.cidade);
    this.formGroup.get('numero').setValue(autor.numero);
    this.formGroup.get('complemento').setValue(autor.complemento);
    this.formGroup.get('email').setValue(autor.email);
    this.formGroup.get('telefone').setValue(autor.telefone);
    this.formGroup.get('obsTelefone').setValue(autor.obsTelefone);
    this.formGroup.get('outroTelefone').setValue(autor.outroTelefone);
    this.formGroup.get('obsOutroTelefone').setValue(autor.obsOutroTelefone);
  }

  deletar(item: AutorModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.autorService.excluir(item.id)
        .pipe(
          finalize(() => {
            this.isAdicionar = false;
            this.isEditar = false;
          })
        )
        .subscribe(
          (response) => console.log('response --->', response),
          (error: ErrorModel) => this.dialogService.showAlert(error.message));
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }


  adicionar(): void {

    this.formGroup.reset();
    this.displayType = {
      titulo: 'Adicionar Autor',
      icone: 'person_add',
    };
    this.isAdicionar = true;
  }

  private montarAutorDemanda(): AutorModel[] {
    const autores: AutorModel[] = [
      {
        id: this.autorId,
        cns: this.formGroup.value.cns,
        cpf: this.formGroup.value.cpf,
        rg: this.formGroup.value.rg,
        dataNascimento: this.formGroup.value.dataNascimento,
        sexo: this.formGroup.value.sexo,
        nome: this.formGroup.value.nome,
        nomeMae: this.formGroup.value.nomeMae,
        cep: this.formGroup.value.cep,
        bairro: this.formGroup.value.bairro,
        cidade: {
          id: this.formGroup.value.cidade ? this.formGroup.value.cidade.id : null
        },
        logradouro: this.formGroup.value.logradouro,
        numero: this.formGroup.value.numero,
        complemento: this.formGroup.value.complemento,
        telefone: this.formGroup.value.telefone,
        obsTelefone: this.formGroup.value.telefone,
        outroTelefone: this.formGroup.value.outroTelefone,
        obsOutroTelefone: this.formGroup.value.obsOutroTelefone,
        email: this.formGroup.value.email,
      }
    ];
    return autores;
  }

  private request(): Observable<AutorModel[]> {
    const postData = this.montarAutorDemanda();
    console.log('postData --->', postData);
    if (this.autorId) {
      return this.autorService.alterar(<AutorModel[]>{
        ...postData,
      }, this.autorId);
    }
    return this.autorService.incluir(postData);
  }

}
