import { AlteracaoDataRetornoFilterModel } from './../../models/alteracao-data-retorno-filter.model';
import { StatusAlteracaoDataRetornoModel } from './../../models/status-alteracao-data-retorno.model';
import { ItemModel } from './../../models/item.model';
import { AlteracaoDataRetornoModel } from './../../models/alteracao-data-retorno.model';
import { AlteracaoDataRetornoService } from './../../services/alteracao-data-retorno.service';
import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';
import { Component, OnInit, ViewChild, ChangeDetectorRef, OnDestroy } from '@angular/core';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { MatPaginator, MatTabGroup, PageEvent } from '@angular/material';
import { Router } from '@angular/router';
import { FormGroup, FormBuilder } from '@angular/forms';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { Observable, Subscription } from 'rxjs';
import { tap, finalize } from 'rxjs/operators';

import { DialogService, LayoutService } from '@datasus/layout';
import { ErrorModel, requestItemsParams, PageableResponseModel, DominioItemModel } from '@datasus/core';


@Component({
  selector: 'scodes-list',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class ListComponent implements OnInit, OnDestroy {

  @ViewChild('paginator') paginator: MatPaginator;

  public filtro: AlteracaoDataRetornoFilterModel;

  public items: AlteracaoDataRetornoModel[] = [];
  public infinityItems: AlteracaoDataRetornoModel[] = [];
  public displayedColumns: string[];

  public onMobile$: Observable<boolean>;

  public formGroup: FormGroup;

  // Data Table Controls (begin)
  public loading = false;
  public infinityItemsMessage = '';
  public total = 0;
  public pageSize = 10;
  // Data Table Controls (end)

  // View Controls (begin)
  public viewChars = false;
  public actionButtonCtrl: Map<ItemModel, boolean> = new Map();
  // View Controls (end)

  public listaStatus: StatusAlteracaoDataRetornoModel[];
  public listaCategorias: DominioItemModel[];
  public listaGrupos: DominioItemModel[];
  public listaProdutos: DominioItemModel[];
  public listaEspecificacoes: DominioItemModel[];

  private scrollDispatcherSubscription: Subscription;

  public readonly ID_STATUS_DEVE_SER_EXECUTADO = 1;
  public readonly ID_STATUS_EXECUTADO = 3;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private alteracaoDataRetornoService: AlteracaoDataRetornoService
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listar Data Retorno');
    this.construirFormulario();
    this.getListaStatus();
    
    this.setOnMobile();
    this.setDisplayedColumns();

  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = {
      ...this.formGroup.value,
      idStatus: this.formGroup.value.status
    };
    this.loading = true;
    this.infinityItemsMessage = '';

    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }

    this.alteracaoDataRetornoService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({
        to: {pageSize: this.pageSize},
        changingPageSizeDefaultTo: this.pageSize,
        withPageEvent: event,
        // withSort: {active: 'titulo', direction: 'asc'},
      }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [ ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<AlteracaoDataRetornoModel>) => {

        this.total = page.total;
        this.items = page.items;

        if (!this.items) {
          return this.dialogService.showNotification('Nenhum registro encontrado.');
        }

        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  // Item Actions (begin)
  edit(item: ItemModel) {
    this.router.navigate([`/rotina/alteracao-data-retorno/editar/${item.id}`]);
  }

  detailsItem(item: ItemModel) {
    this.router.navigate([`/rotina/alteracao-data-retorno/detalhar/${item.id}`]);
  }

  delete(item: ItemModel) {
    const buttons: Map<string, () => void> = new Map();
    
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.alteracaoDataRetornoService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });

    buttons.set('Não', () => console.log('Abortou a ação #2', item));
    
    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });

  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
        '';
  }

  private construirFormulario(): void {
    this.formGroup = this.formBuilder.group({
      dataInicio: [null],
      dataFim: [null],
      status: [null],
      titulo: [null],
    });
  }

  // TODO: Refactoring
  private getListaStatus() {
    this.alteracaoDataRetornoService.obterStatus().subscribe(resp => {
      this.listaStatus = [
        { id: null, descricao: 'Todos' },
        ...resp
      ];
    });
    
    /*
    
    this.listaStatus = [
      { codigo: 'todos', descricao: 'Todos'},
      { codigo: 'deve-ser-executado', descricao: 'Deve ser executado'},
      { codigo: 'executando', descricao: 'Executando'},
      { codigo: 'ja-executado', descricao: 'Já executado'},
    ];

    */
  }

  // Item Actions (end)
  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

  private setOnMobile(): void {
    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
  }

  private setDisplayedColumns(): void {
    this.displayedColumns = [
      'dataCadastro',
      'titulo',
      'status',
      'actions',
    ];
  }
}
