import { Component, OnInit, ViewChild, ChangeDetectorRef } from '@angular/core';
import { MatPaginator, MatTabGroup, PageEvent } from '@angular/material';
import { Observable, Subscription } from 'rxjs';
import { finalize, tap } from 'rxjs/operators';
import { FormGroup, FormBuilder } from '@angular/forms';
import { Router } from '@angular/router';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { trigger, transition, style, animate, query } from '@angular/animations';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel } from '@datasus/core';

import { DemandaModel } from 'src/app/demandas/models/demanda.model';
import { InativarDemandaModel } from '../../models/inativar-demanda.model';
import { InativarDemandaService } from '../../services/inativar-demanda.service';
import { StatusModel } from 'src/app/shared/models/status.model';

@Component({
  selector: 'scodes-inativar-demanda-list',
  templateUrl: './inativar-demanda-list.component.html',
  styleUrls: ['./inativar-demanda-list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class InativarDemandaListComponent implements OnInit {

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: InativarDemandaModel[] = [];
  infinityItems: InativarDemandaModel[] = [];

  listaStatus: StatusModel;

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<DemandaModel, boolean> = new Map();
  // View Controls (end)


  // Data Table Config
  displayedColumns: string[] = [
    'dataCadastro',
    'titulo',
    'rotina',
    'parametro',
    'actions'
  ];

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private inativarDemandaService: InativarDemandaService,

  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Inativar Demandas');
    this.construirFormulario();


    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
  }

  construirFormulario(): void {
    this.formGroup = this.formBuilder.group({
      dataInicio: [null],
      dataFim: [null],
      status: [null],
      titulo: [null],
    });
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.inativarDemandaService.obterTodosPaginado({
      filter: this.formGroup.value,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<InativarDemandaModel>) => {
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  editar(item: InativarDemandaModel): void {

  }

  deletar(item: InativarDemandaModel): void {

  }


  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

}
