import { Component, OnInit, ChangeDetectorRef, ViewChild } from '@angular/core';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Router } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { MatPaginator, MatTabGroup, PageEvent } from '@angular/material';

import { Observable, Subscription } from 'rxjs';
import { tap, finalize } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel, DominioItemModel, download, stringToBlob } from '@datasus/core';

import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';
import { ArquivoBase64Model } from 'src/app/shared/models/arquivo-base64.model';
import { UnidadeDispensadoraService } from 'src/app/shared/services/unidade-dispensadora.service';

import { InventarioFilterModel } from '../../models/inventario-filter.model';
import { InventarioModel } from '../../models/inventario.model';
import { InventarioService } from '../../services/inventario.service';

@Component({
  selector: 'scodes-inventario-list',
  templateUrl: './inventario-list.component.html',
  styleUrls: ['./inventario-list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class InventarioListComponent implements OnInit {

  filtro: InventarioFilterModel;
  listaStatus: any[] = [
    { codigo: null, descricao: 'Todos' },
    { codigo: true, descricao: 'Em Andamento' },
    { codigo: false, descricao: 'Finalizado' },
  ];

  listaUnidadeDispensadora: DominioItemModel[];

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: InventarioModel[] = [];
  infinityItems: InventarioModel[] = [];

  // Data Table Config
  displayedColumns: string[] = [
    'dataCadastro',
    'nomeInventario',
    'ud',
    'status',
    'actions',
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<InventarioModel, boolean> = new Map();
  // View Controls (end)

  minDate: Date;

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private inventarioService: InventarioService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Listagem de Inventários');

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

    this.formGroup = this.formBuilder.group({
      idUnidadeDispensadora: [null],
      situacaoAtivo: [null],
      dataInicio: [null],
      dataFim: [null],
    });

    this.formGroup.get('dataInicio').valueChanges
      .subscribe((value) => this.minDate = value);

    this.unidadeDispensadoraService
      .obterDominio()
      .subscribe(
        (response) => this.listaUnidadeDispensadora = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );

  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = this.formGroup.value;
    this.loading = true;
    this.infinityItemsMessage = '';

    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }

    this.inventarioService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({
        to: {pageSize: this.pageSize},
        changingPageSizeDefaultTo: this.pageSize,
        withPageEvent: event,
        withSort: {active: 'unidadeDispensadora', direction: 'asc'},
      }),
    }).pipe(finalize(() => this.loading = false)
    ).subscribe(
      (page: PageableResponseModel<InventarioModel>) => {
        this.total = page.total;
        this.items = page.items;

        if (!this.items) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }

        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  // Item Actions (begin)
  abrirItem(item: InventarioModel): void {
    this.router.navigate([`/rotina/inventario/item/${item.id}`]);
  }

  gerarRelatorioItem(item: InventarioModel): void {
    this.loading = true;
    this.inventarioService.relatorioItem(item.id)
      .pipe(finalize(() => this.loading = false))
      .subscribe(
        (arquivo: ArquivoBase64Model) => {
          download(stringToBlob(atob(arquivo.base64), arquivo.mimeType), arquivo.filename);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }

  gerarRelatorioFinal(item: InventarioModel): void {
    this.loading = true;
    this.inventarioService.relatorioConsolidado(item.id)
      .pipe(finalize(() => this.loading = false))
      .subscribe(
        (arquivo: ArquivoBase64Model) => {
          download(stringToBlob(atob(arquivo.base64), arquivo.mimeType), arquivo.filename);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

}
