import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel } from '@datasus/core';

import { ModeloRelatorioTecnicoModel } from '../../../../../shared/models/modelo-relatorio-tecnico.model';
import { ModeloRelatorioTecnicoService } from '../../../../../shared/services/modelo-relatorio-tecnico.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { TipoModeloModel } from 'src/app/shared/models/tipo-modelo.model';
import {MensagemEnum} from '../../../../../shared/enums/mensagem.enum';


@Component({
  selector: 'scodes-modelo-relatorio-tecnico-form',
  templateUrl: './modelo-relatorio-tecnico.form.component.html',
  styleUrls: ['./modelo-relatorio-tecnico.form.component.scss'],
})
export class ModeloRelatorioTecnicoFormComponent implements OnInit {

  descricao = '';
  formGroup: FormGroup;
  saving: boolean;
  listaTipoDemanda: DominioItemModel[];
  listaInativo = [
    { label: 'Sim', value: true },
    { label: 'Não', value: false }
  ];

  private id: number;
  private readonly tipoModelo: TipoModeloModel = { id: 2};



  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private modeloService: ModeloRelatorioTecnicoService,
    private tipoDemandaService: TipoDemandaService
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de modelo');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descritor: [null, [Validators.required]],
      situacaoInativo: [true],
      tipoDemanda: [null, [Validators.required]]
    });

    this.tipoDemandaService.obterDominio(2).subscribe(
      (response: DominioItemModel[]) => this.listaTipoDemanda = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );

    this.route.data.pipe(
      take(1),
      map((data: { item: ModeloRelatorioTecnicoModel }) => data.item),
      filter((item: ModeloRelatorioTecnicoModel) => !!item),
      tap((item: ModeloRelatorioTecnicoModel) => this.editItem(item)),
    ).subscribe();
  }

  editItem(item: ModeloRelatorioTecnicoModel): void {

    this.layoutService.setTitle(`Alteração de Relatório Técnico`);
    this.id = item.id;
    this.formGroup.get('descritor').setValue(item.descritor);
    this.formGroup.get('tipoDemanda').setValue(item.tipoDemanda.id.toString());
    this.formGroup.get('situacaoInativo').setValue(item.situacaoInativo);
    this.descricao = item.modelo;
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/rotina/tabela/modelo-relatorio-tecnico/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  resetForm(event: MouseEvent) {
    setTimeout(() => {
      this.formGroup.get('situacaoInativo').setValue(true);
    });
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }

  private request(): Observable<ModeloRelatorioTecnicoModel> {
    const postData: ModeloRelatorioTecnicoModel  = {
      descritor: this.formGroup.value.descritor,
      situacaoInativo: this.formGroup.value.situacaoInativo,
      modelo: this.descricao,
      tipoDemanda: {
        id: this.formGroup.value.tipoDemanda,
      },
      tipoModelo: this.tipoModelo,
    };

    if (this.id) {
      return this.modeloService.alterar(<ModeloRelatorioTecnicoModel>{
        id: this.id,
        ...postData,
      }, this.id);
    }
    return this.modeloService.incluir(postData);
  }
}
