import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Router } from '@angular/router';
import { PageEvent, MatPaginator } from '@angular/material/paginator';
import { MatTabGroup } from '@angular/material';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { Observable, Subscription } from 'rxjs';
import { tap, finalize } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel, DominioItemModel } from '@datasus/core';

import { ModeloRelatorioTecnicoModel } from '../../../../../shared/models/modelo-relatorio-tecnico.model';
import { ModeloRelatorioTecnicoService } from '../../../../../shared/services/modelo-relatorio-tecnico.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';

@Component({
  selector: 'scodes-modelo-relatorio-tecnico-list',
  templateUrl: './modelo-relatorio-tecnico.list.component.html',
  styleUrls: ['./modelo-relatorio-tecnico.list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class ModeloRelatorioTecnicoListComponent implements OnInit, OnDestroy {

  filtro: ModeloRelatorioTecnicoModel;
  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: ModeloRelatorioTecnicoModel[] = [];
  infinityItems: ModeloRelatorioTecnicoModel[] = [];
  listaTipoDemanda: DominioItemModel[];

  // Data Table Config
  displayedColumns: string[] = [
    'descritor',
    'inativo',
    'tipo',
    'actions',
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<ModeloRelatorioTecnicoModel, boolean> = new Map();
  // View Controls (end)

  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private modeloService: ModeloRelatorioTecnicoService,
    private tipoDemandaService: TipoDemandaService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listagem Modelo Relatório Técnico');

    this.tipoDemandaService.obterDominio().subscribe(
      (response: DominioItemModel[]) => this.listaTipoDemanda = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
    this.formGroup = this.formBuilder.group({
      descritor: [null],
      tipoDemandaId: [null]
    });
  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = this.formGroup.value;
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.modeloService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<ModeloRelatorioTecnicoModel>) => {
        console.log(' --->', page);
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  // Item Actions (begin)
  view(item: ModeloRelatorioTecnicoModel) {
    this.router.navigate(['/rotina/tabela/modelo-relatorio-tecnico/visualizar', item.id]);
  }

  edit(item: ModeloRelatorioTecnicoModel) {
    this.router.navigate(['/rotina/tabela/modelo-relatorio-tecnico/editar', item.id]);
  }

  delete(item: ModeloRelatorioTecnicoModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.modeloService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }
}
