import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { PlanoSaudeModel } from '../../models/plano-saude.model';
import { PlanoSaudeService } from '../../services/plano-saude.service';
import {MensagemEnum} from '../../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-plano-saude-form',
  templateUrl: './plano-saude.form.component.html',
  styleUrls: ['./plano-saude.form.component.scss'],
})
export class PlanoSaudeFormComponent implements OnInit {

  formGroup: FormGroup;
  saving: boolean;

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private planoSaudeService: PlanoSaudeService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Plano de Saúde');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descricao: [
        null,
        Validators.compose([
          Validators.maxLength(50),
          Validators.required,
        ])
      ],
      tipo: [
        null,
        Validators.compose([
          Validators.maxLength(1),
          Validators.required,
        ])
      ],
    });

    this.route.data.pipe(
      take(1),
      map((data: { item: PlanoSaudeModel }) => data.item),
      filter((item: PlanoSaudeModel) => !!item),
      tap((item: PlanoSaudeModel) => this.editItem(item)),
    ).subscribe();
  }

  editItem(item: PlanoSaudeModel): void {
    this.layoutService.setTitle(`Alteração de Plano de Saúde`);
    this.id = item.id;
    this.formGroup.get('descricao').setValue(item.descricao);
    this.formGroup.get('tipo').setValue(item.tipo);
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['rotina/tabela/plano-saude/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private request(): Observable<PlanoSaudeModel> {
    if (this.id) {
      return this.planoSaudeService.alterar(<PlanoSaudeModel>{
        id: this.id,
        ...this.formGroup.value,
      }, this.id);
    }
    return this.planoSaudeService.incluir(this.formGroup.value);
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }
}
