import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { finalize, take, map, filter, tap } from 'rxjs/operators';
import { Observable, Subscription } from 'rxjs';
import { MatPaginator } from '@angular/material';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';

import { LayoutService, DialogService } from '@datasus/layout';
import { DominioItemModel, ErrorModel, requestItemsParams, PageableResponseModel } from '@datasus/core';

import { EspecificacaoService } from '../../services/especificacao.service';
import { ProdutoModel } from '../../models/produto.model';
import { EspecificacaoModel } from '../../models/especificacao.model';
import { GrupoEspecificacaoService } from '../../services/grupo-especificacao.service';
import { ApresentacaoService } from '../../services/apresentacao.service';
import { ItemService } from '../../../../../shared/services/item.service';
import { ItemModel } from '../../models/item.model';

@Component({
  selector: 'scodes-especificacao-form',
  templateUrl: './especificacao-form.component.html',
  styleUrls: ['./especificacao-form.component.scss']
})
export class EspecificacaoFormComponent implements OnInit, OnDestroy {

  formGroup: FormGroup;
  saving: boolean;

  listaGrupos: DominioItemModel[];
  listaApresentacoes: DominioItemModel[];
  produto: ProdutoModel;

   // Data Table Config
   displayedColumns: string[] = [
    'tipo',
    'marca',
    'siafisico',
    'actions',
  ];

  filtro: any;

  // Retrived Data Source
  items: ItemModel[] = [];
  infinityItems: ItemModel[] = [];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

   // Data Table Controls (begin)
   loading = false;
   infinityItemsMessage = '';
   total = 0;
   pageSize = 0;
   // Data Table Controls (end)

   // View Controls (begin)
   viewChars = false;
   actionButtonCtrl: Map<EspecificacaoModel, boolean> = new Map();
   // View Controls (end)

  private scrollDispatcherSubscription: Subscription;

  idProduto: number;
  idEspecificacao: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private especificacaoService: EspecificacaoService,
    private grupoEspecificacaoService: GrupoEspecificacaoService,
    private apresentacaoService: ApresentacaoService,
    private changeDetectorRef: ChangeDetectorRef,
    private scrollDispatcher: ScrollDispatcher,
    private itemService: ItemService,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Cadastro da Especificação');

    this.formGroup = this.formBuilder.group({
      especificacao: [null, [Validators.required]],
      grupo: [null, [Validators.required]],
      apresentacao: [null, [Validators.required]]
    });

    this.grupoEspecificacaoService
    .obterDominio(this.idProduto)
    .subscribe(
      (response) => this.listaGrupos = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.apresentacaoService
      .obterDominio()
      .subscribe(
        (response) => this.listaApresentacoes = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message));

    this.route.data.pipe(
      take(1),
      map((data: { produto: ProdutoModel }) => data.produto),
      filter((produto: ProdutoModel) => !!produto),
      tap((produto: ProdutoModel) => this.getProduto(produto)),
    ).subscribe();

    this.route.data.pipe(
      take(1),
      map((data: { especificacao: EspecificacaoModel }) => data.especificacao),
      filter((especificacao: EspecificacaoModel) => !!especificacao),
      tap((especificacao: EspecificacaoModel) => this.getEspecificacao(especificacao)),
    ).subscribe();

  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  getProduto(produto: ProdutoModel): void {
    this.idProduto = produto.id;
    this.produto = produto;
  }

  getEspecificacao(especificacao: EspecificacaoModel): void {
    this.layoutService.setTitle(`Alteração da Especificação`);
    this.idEspecificacao = especificacao.id;
    this.formGroup.get('especificacao').setValue(especificacao.nome);
    this.formGroup.get('grupo').setValue(`${especificacao.grupoEspecificacao.id}`);
    this.formGroup.get('apresentacao').setValue(`${especificacao.apresentacao.id}`);
    this.requestItems(this.idProduto, this.idEspecificacao);
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.salvarEspecificacao().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      (resp) => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate([`/rotina/tabela/produto/${resp.produto.id}/especificacao/${resp.id}/item/inserir`]);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }


  // Data Events (begin)

  requestItems(idProduto: number, idEspecificacao: number): void {
    this.filtro = { idProduto: idProduto, idEspecificacao: idEspecificacao };
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.itemService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<ItemModel>) => {
        console.log('page -->', page);
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  novoItem(): void {
    this.router.navigate([`/rotina/tabela/produto/${this.idProduto}/especificacao/${this.idEspecificacao}/item/inserir`]);
  }

  // Item Actions (begin)
  edit(item: ItemModel): void {
    this.router.navigate([`/rotina/tabela/produto/${this.idProduto}/especificacao/${this.idEspecificacao}/item/${item.id}/editar`]);
  }

  delete(item: ItemModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.itemService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems(this.idProduto, this.idEspecificacao);
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }



  private salvarEspecificacao(): Observable<EspecificacaoModel> {
    const obj: EspecificacaoModel = {
      apresentacao: {id: Number(this.formGroup.value.apresentacao)},
      grupoEspecificacao: {id: Number(this.formGroup.value.grupo)},
      nome: this.formGroup.value.especificacao,
      produto: {id: this.idProduto}
    };

    if (this.idEspecificacao) {
      return this.especificacaoService.alterar(<EspecificacaoModel>{
        id: this. idEspecificacao,
        ...obj,
      }, this. idEspecificacao);
    }
    return this.especificacaoService.incluir(obj);
  }

}

