import { Component, OnInit, ChangeDetectorRef, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { MatPaginator, MatTabGroup, PageEvent } from '@angular/material';
import { Observable, Subscription } from 'rxjs';
import { tap, finalize, debounceTime, switchMap, take } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, requestItemsParams, PageableResponseModel, DominioItemModel } from '@datasus/core';

import { SimilaridadeModel } from '../../models/similaridade.model';
import { ProdutoService } from '../../../produto/services/produto.service';
import { ItemService } from '../../../../../shared/services/item.service';
import { ItemModel } from '../../../produto/models/item.model';
import { ItemFilterModel } from '../../../produto/models/item-filter.model';
import { MensagemEnum } from 'src/app/shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-similaridade-list',
  templateUrl: './similaridade-list.component.html',
  styleUrls: ['./similaridade-list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class ProdutoListComponent implements OnInit {

  filtro: ItemFilterModel;
  listaProdutos: DominioItemModel[];

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: ItemModel[] = [];
  infinityItems: ItemModel[] = [];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 10;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<SimilaridadeModel, boolean> = new Map();
  // View Controls (end)

  // Data Table Config
  displayedColumns: string[] = [
    'codigo',
    'produto',
    'actions',
  ];


  private scrollDispatcherSubscription: Subscription;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private produtoService: ProdutoService,
    private itemService: ItemService,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Listagem de Similaridade');

    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );

    this.formGroup = this.formBuilder.group({
      produto: [null],
      codigo: [null],
    });

    this.formGroup.get('produto').valueChanges
    .pipe(
      debounceTime(500),
      switchMap((value) => {
        if (typeof value === 'string') {
          return this.produtoService.obterAutocompleteDominioPorTermo(value);
        }
        return [];
      }),
      )
    .subscribe((response: DominioItemModel[] ) => {
      if (response.length >= 1) {
        this.listaProdutos = response;
      }
    });

  }

  // Item Actions (begin)
  visualizarSimilaridade(item: SimilaridadeModel) {
    this.router.navigate([`/rotina/tabela/similaridade/${item.id}/item`]);
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification(MensagemEnum.FORM_INVALID);
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = {
      codigo: this.formGroup.value.codigo,
      idProduto: this.formGroup.value.produto ? this.formGroup.value.produto.codigo : null
    };

    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.itemService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({
        to: this,
        changingPageSizeDefaultTo: this.pageSize,
        withPageEvent: event,
        withSort: {active: 'id', direction: 'asc'},
      }),
    }).pipe(
      finalize(() => {
        this.loading = false;
      })
    ).subscribe(
      (page: PageableResponseModel<ItemModel>) => {
        this.total = page.total;
        this.items = page.items;

        if (!this.items || this.items.length === 0) {
          return this.dialogService.showNotification(MensagemEnum.NO_RESULTS_FOUND);
        }

        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  displayFn(produtoDominio?: DominioItemModel): string | undefined {
    return produtoDominio ? produtoDominio.descricao : undefined;
  }


  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

}
