import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, tap, finalize, take, filter } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { UnidadeTransferenciaModel } from '../../models/unidade-transferencia.model';
import { UnidadeTransferenciaService } from '../../services/unidade-transferencia.service';
import {MensagemEnum} from '../../../../../shared/enums/mensagem.enum';

@Component({
  selector: 'scodes-unidade-transferencia-form',
  templateUrl: './unidade-transferencia.form.component.html',
  styleUrls: ['./unidade-transferencia.form.component.scss'],
})
export class UnidadeTransferenciaFormComponent implements OnInit {

  formGroup: FormGroup;
  saving: boolean;

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private tipoAcaoService: UnidadeTransferenciaService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Cadastro de Unidade de Transferência');
    this.id = null;

    this.formGroup = this.formBuilder.group({
      descricao: [
        null,
        Validators.compose([
            Validators.maxLength(50),
            Validators.required,
        ])
      ],
    });

    this.route.data.pipe(
      take(1),
      map((data: {item: UnidadeTransferenciaModel}) => data.item),
      filter((item: UnidadeTransferenciaModel) => !!item),
      tap((item: UnidadeTransferenciaModel) => this.editItem(item)),
    ).subscribe();
  }

  editItem(item: UnidadeTransferenciaModel): void {
    this.layoutService.setTitle(`Alteração de Unidade de Transferência`);
    this.id = item.id;
    this.formGroup.get('descricao').setValue(item.descricao);
  }

  save(event: Event): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/rotina/tabela/unidade-transferencia/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private request(): Observable<UnidadeTransferenciaModel> {
    if (this.id) {
      return this.tipoAcaoService.alterar(<UnidadeTransferenciaModel>{
        id: this.id,
        ...this.formGroup.value,
      }, this.id);
    }
    return this.tipoAcaoService.incluir(this.formGroup.value);
  }

  isInvalid(formControlName: string): boolean {
    return this.formGroup.get(formControlName).invalid;
  }

  getErrorMessage(formControlName: string): string {
    const fc = this.formGroup.get(formControlName);
    return fc.hasError('required') ? MensagemEnum.REQUIRED :
      '';
  }
}
