import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Router } from '@angular/router';
import { PageEvent, MatPaginator } from '@angular/material/paginator';
import { MatTabGroup, MatAutocompleteSelectedEvent } from '@angular/material';
import { ScrollDispatcher, CdkScrollable } from '@angular/cdk/overlay';
import { Observable, Subscription } from 'rxjs';
import { tap, finalize, debounceTime, switchMap } from 'rxjs/operators';

import { LayoutService, DialogService } from '@datasus/layout';
import { requestItemsParams, PageableResponseModel, ErrorModel } from '@datasus/core';

import { VaraModel } from '../../../../../shared/models/vara.model';
import { VaraService } from '../../../../../shared/services/vara.service';
import { UfService } from 'src/app/shared/services/uf.service';
import { CidadeService } from 'src/app/shared/services/cidade.service';
import { UfModel } from 'src/app/shared/models/uf.model';
import { CidadeModel } from 'src/app/shared/models/cidade.model';

@Component({
  selector: 'scodes-vara-list',
  templateUrl: './vara.list.component.html',
  styleUrls: ['./vara.list.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class VaraListComponent implements OnInit, OnDestroy {

  filtro: VaraModel;
  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

  // Retrived Data Source
  items: VaraModel[] = [];
  infinityItems: VaraModel[] = [];

  // Data Table Config
  displayedColumns: string[] = [
    'descricao',
    'actions',
  ];

  // Viewport Observable
  onMobile$: Observable<boolean>;

  // Form Group
  formGroup: FormGroup;

  // Data Table Controls (begin)
  loading = false;
  infinityItemsMessage = '';
  total = 0;
  pageSize = 0;
  // Data Table Controls (end)

  // View Controls (begin)
  viewChars = false;
  actionButtonCtrl: Map<VaraModel, boolean> = new Map();
  // View Controls (end)

  private scrollDispatcherSubscription: Subscription;

  listaUfs: UfModel[];
  listaCidades: CidadeModel[] = [];

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private scrollDispatcher: ScrollDispatcher,
    private changeDetectorRef: ChangeDetectorRef,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private modeloService: VaraService,
    private ufService: UfService,
    private cidadeService: CidadeService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Listagem de Vara');
    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
    this.formGroup = this.formBuilder.group({
      descricao: [null],
      estado: [null, [Validators.required]],
      cidade: [null, [Validators.required]]
    });

    this.ufService.obterTodos()
    .subscribe(
      (response) => this.listaUfs = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );

    this.formGroup.get('estado').valueChanges
    .pipe(
      tap(() => this.formGroup.get('cidade').setValue(null)),
      switchMap((uf: UfModel) => this.cidadeService.obterCidadesPorUf(uf.id)),
    ).subscribe(
      (resp: any) => this.listaCidades = resp);

    this.formGroup.get('cidade').valueChanges
    .pipe(debounceTime(350))
    .subscribe(
      (cidade) => {
        if (typeof cidade === 'string') {
          return this.cidadeService.obterCidadesPorNome(cidade, this.formGroup.get('estado').value.id)
          .subscribe(
              (response) => this.listaCidades = response,
              (error: ErrorModel) => this.dialogService.showNotification(error.message));
        }
      });
  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
  }

  // Data Events (begin)
  search(event: Event, tabs?: MatTabGroup): void {
    event.preventDefault();
    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }
    if (tabs) {
      tabs.selectedIndex = 1;
    }
    this.infinityItems = [];
    this.requestItems();
  }

  requestItems(event?: PageEvent): void {
    this.filtro = this.formGroup.value;
    this.loading = true;
    this.infinityItemsMessage = '';
    if (this.scrollDispatcherSubscription) {
      this.changeDetectorRef.detectChanges();
    }
    this.modeloService.obterTodosPaginado({
      filter: this.filtro,
      ...requestItemsParams({ to: this, withPageEvent: event }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page: PageableResponseModel<VaraModel>) => {
        console.log(' --->', page);
        this.total = page.total;
        this.items = page.items;
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }
  // Data Events (end)

  // Item Actions (begin)
  view(item: VaraModel) {
    this.router.navigate(['/rotina/tabela/vara/visualizar', item.id]);
  }

  edit(item: VaraModel) {
    this.router.navigate(['/rotina/tabela/vara/editar', item.id]);
  }

  delete(item: VaraModel) {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('Sim', () => {
      this.layoutService.loading();
      console.log('Ação #2', item);
      this.modeloService.excluir(item.id).pipe(
        finalize(() => this.layoutService.loaded()),
      ).subscribe(
        () => {
          this.dialogService.showNotification('Registro excluído com sucesso');
          this.requestItems();
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message),
      );
    });
    buttons.set('Não', () => console.log('Abortou a ação #2', item));

    this.dialogService.showConfirmation({
      question: 'Deseja mesmo remover?',
      title: 'Cuidado!',
      anwsers: buttons,
      type: 'warn',
    });
  }
  // Item Actions (end)

  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

  obterCidades(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidade').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  displayFn(cidadeDominio?: CidadeModel): string | undefined {
    return cidadeDominio ? cidadeDominio.descricao : undefined;
  }
}
